/*
 * adc_simple_avr.c
 *
 * Einfache ADC-Routinen fuer Testzwecke. ADC wird gepollt.
 *
 * Created: 18.02.2014
 *  Author: DL1DOW
 */



#include <avr/io.h>
#include "../../ioMapping.h"
#include "adc_simple.h"

#ifndef AVR
    #error "unknown MCU"
#endif




// Settings ADC
// Referenzspannung: AVcc mit externem, Kondensator an ARef,
// Ergebnis rechtsbndig ausrichten:
#define ADCMUXCONFIG (1<<REFS0)|(0<<REFS1)|(0 << ADLAR)

#define ADCMUXCHANNEL 4


/* Initialisierung ADC
 * Konfiguration ist oben festgelegt, deshalb ist der Uebergabeparamter nur ein
 * Dummy. Rueckgabewert ebenfalls Dummy */
uint_fast8_t adc_simple_init(ADC_TypeDef *ADCx)
{
	(void) ADCx;

	// ADC initialisieren
	while (bit_is_set(ADCSRA, ADSC));
	ADMUX = ADCMUXCONFIG;

	// Datenblatt S.256
	// ADC einschalten
	// 16 MHz / 0,2 MHz = 80 -> Teilungsfaktor 128 // ADC einschalten
	// 16 MHz / 128 = 125 kHz ADC-Clock
	ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);

	return EXIT_SUCCESS;
}


/* Der ADC braucht nicht deinitialisiert zu werden. Nur ein Dummy. */
uint_fast8_t adc_simple_deinit(ADC_TypeDef *ADCx)
{
	(void) ADCx;
	return EXIT_SUCCESS;
}


/* ADC-Kanal auslesen (10 Bit, rechtsbuendig)
 * Routine sehr langsam, da warten
 * channel: 0 - 6
 * Konstante ADCMUXCONFIG
 * Seite 225 Datenblatt
 *
 * *ADCx : Dummy
 * return: ADC-Sample
 * */
uint16_t adc_simple_getval(ADC_TypeDef *ADCx)
{
	(void) ADCx;

	// Erst ADCL, dann ADCH auslesen. Danach ist das Register geloescht.
	// hier wird nur ADCH benoetigt und ADCL ignoriert:
	uint8_t muxChannel = ADCMUXCHANNEL;

	ADMUX = ADCMUXCONFIG | muxChannel;
	ADCSRA  |= (1 << ADSC);   // Wandlung starten
	while (bit_is_set(ADCSRA, ADSC)); // warten
	return ADC;
}


