/*
 * adc_simple_stm32.c
 *
 * Einfache ADC-Routinen fuer Testzwecke. Freilaufender ADC, letzter Wert
 * wird gepollt.
 *
 * Created: 18.02.2014
 *  Author: DL1DOW
 */



#include <stdio.h>
#include <stdlib.h>
#include "../src_avrcom/io.h"
#include "../../ioMapping.h"


#ifdef STM32F10x
    #include "stm32f10x_adc.h"
    #include "stm32f10x_gpio.h"
    #include "stm32f10x_rcc.h"
#elif defined(STM32F4xx)
    #error "only for STM32F10x implemented yet"
#else
    #error "unknown MCU"
#endif


/* Hilfsfunktion: Kalibrierung vornehmen */
static void adc_calib(ADC_TypeDef *ADCx)
{
	ADC_Cmd(ADCx,ENABLE);
	ADC_ResetCalibration(ADCx);
	while(ADC_GetResetCalibrationStatus(ADCx));
	ADC_StartCalibration(ADCx);
	while(ADC_GetCalibrationStatus(ADCx));
	ADC_SoftwareStartConvCmd(ADCx,ENABLE);
}


/* Initialisierung ADC
 *
 * *ADCx : ADC1 oder ADC2
 * return: Errorcode  */
uint_fast8_t adc_simple_init(ADC_TypeDef *ADCx)
{
	ADC_InitTypeDef  ADC_InitStructure;
	GPIO_InitTypeDef GPIO_InitStructure;

	// ADC-Takt max. 14MHz, hier 72MHz / 6 = 12MHz
	RCC_ADCCLKConfig(RCC_PCLK2_Div6);

	initGpioClock(GPIOC);
	initAfioClock();

	if (ADCx==ADC1)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1,ENABLE);
	else if(ADCx==ADC2)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC2,ENABLE);
	else
		return EXIT_FAILURE;

	GPIO_InitStructure.GPIO_Pin = POTI_UPDWN_Pin;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(POTI_UPDWN_GPIO,&GPIO_InitStructure);

	ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
	ADC_InitStructure.ADC_DataAlign          = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_ExternalTrigConv   = ADC_ExternalTrigInjecConv_None;
	ADC_InitStructure.ADC_Mode               = ADC_Mode_Independent;
	ADC_InitStructure.ADC_NbrOfChannel       = 1;
	ADC_InitStructure.ADC_ScanConvMode       = DISABLE;
	ADC_Init(ADCx,&ADC_InitStructure);

	ADC_RegularChannelConfig(ADCx,POTI_UPDWM_ADCCH,1,ADC_SampleTime_1Cycles5);

	ADC_Cmd(ADCx,ENABLE);

	ADC_SoftwareStartConvCmd(ADCx,ENABLE);

	adc_calib(ADCx);
	return EXIT_SUCCESS;
}


/* Der ADC braucht nicht deinitialisiert zu werden. Nur ein Dummy. */
uint_fast8_t adc_simple_deinit(ADC_TypeDef *ADCx)
{
	(void) ADCx;
	return EXIT_SUCCESS;
}


/* ADC-Kanal auslesem */
uint16_t adc_simple_getval(ADC_TypeDef *ADCx)
{
	return ADC_GetConversionValue(ADCx);
}


