/*
 * delay.h
 *
 * Delays: Hilfsdefinitionen fuer die Kompatibilitaet mit AVR-Code
 *
 * Created: 22.12.2013
 *  Author: Nicolas
 */

#ifndef DELAY_H
    #define DELAY_H
 
    #include "../src_base/datatypes.h"
    #include "../src_avrcom/platform.h"

    #ifdef PLATFORM_AVR
    	// Beim AVR werden die Routinen des AVR-GCC verwendet
    	#include <util/delay.h>
    	#define delay_us _delay_us
		#define delay_ms _delay_ms

	#elif defined(PLATFORM_STM32)
    	// Ersatz fuer delay-Routinen
    	#include <stdint.h>
        #include "../../main.h"


    	#if F_CPU == 24000000ULL
      	    #define TUNE_FAC 0.96154
      	    #define SYSCLK 24000000ULL
              
    	#elif F_CPU == 72000000ULL
      	  	#define TUNE_FAC 1.0
      	  	#define SYSCLK 36000000ULL
                
    	#else
      	  	#error "delay routines not prepared for this System Clock rate"
                
        #endif

    	#define delay_us(delayval) delay_us_( \
        	(uint32_t)( (SYSCLK) / 3000000ULL *delayval * TUNE_FAC) )

		#define delay_ms(delayval) delay_us_( \
        	(uint32_t)( (SYSCLK) / 3000ULL *delayval * TUNE_FAC) )

	  	static inline void delay_us_(uint64_t delayval)
	  	{
	  		if(delayval) 
            {
	  			asm volatile (\
	  						"L_LOOPUS_%=:     \n\t" \
	  						"subs %0, %0, #1  \n\t" \
	  						"bne  L_LOOPUS_%= \n\t" \
	  						:  "+r" (delayval) );
	  		}
	  		else 
            {
	  			asm volatile("nop");
	  		}
	  	}

      #elif defined(PLATFORM_PC)
          /* Matlab Mex-File / PC: Warten mit SDL
           * Es werden nur die langen Wartezeiten wirklich ausgefuehrt.  */  
	  	  void delay_us(uint32_t a);
	  	  void delay_ms(uint32_t a);
            
  	  #else
          #error "No MCU defined"
          
      #endif

#endif // DELAY_H

