/*
 * io.h
 *
 * Hilfsdefinitionen fuer die Kompatibilitaet zwischen STM32- und AVR-Code
 *
 * GPIO-Operationen
 *
 * Created: 22.02.2014
 *  Author: Nicolas
 */


#ifndef AVRCOM_IO_H
    #define AVRCOM_IO_H
	
    #include "../src_avrcom/platform.h"
    
    #ifdef PLATFORM_AVR

	    #include <avr/io.h>

        typedef	volatile uint8_t GPIO_t;
        typedef	uint8_t IOPin_t;

		#include "io_avr.h"

    #elif defined(STM32F10x)

		#include "stm32f10x_gpio.h"

        typedef	GPIO_TypeDef GPIO_t;
        typedef	uint16_t IOPin_t;
	
		#include "../src_base/bitmath.h"
        #include "io_stm32f10x.h"


    #elif defined(STM32F4XX)
	
        #include "stm32f4xx_gpio.h"
        #error "gpio functions not implemented for this MCU"


	#elif defined PLATFORM_PC
		
        #include <stdint.h>
        
        typedef	uint8_t GPIO_t;
        typedef	uint8_t IOPin_t;
        
		// Auf dem PC sind alle Funktionen nur Dummys und werden ignoriert
		#include "io_mock.h"
		
    #else
        #error "unknown Platform"
        
    #endif


#endif // AVRCOM_GPIO_H
