/*
 * io_mock.inc
 *
 * Hilfsdefinitionen fuer die Kompatibilitaet zwischen STM32- und AVR-Code
 *
 * GPIO-Operationen
 *
 * Created: 22.02.2014
 *  Author: DL1DOW
 */


#ifndef AVRCOM_IO_MOCK_H
    #define AVRCOM_IO_MOCK_H
	
    #include "../src_avrcom/platform.h"
    #include "../src_base/datatypes.h"

	#ifndef PLATFORM_PC
		#error "This Header is for PC mockup only"
	#endif
				
		
	// Auf dem PC sind alle Funktionen nur Dummys und werden ignoriert
	static_always_inline void io_setBit(GPIO_t * const GPIOx, const uint16_t Pin)
    {
        (void) GPIOx;
		(void) Pin;
    }


    /* Einzelnes oder mehrere Bits eines Ports loeschen
     *
     * GPIOx
     * Pin: Bitmaske */
    static_always_inline void io_clearBit(GPIO_t * const GPIOx, const uint8_t Pin) 
    {
	    (void) GPIOx;
		(void) Pin;
    }	

	
    /* Einzelnes oder mehrere Bits eines Ports umschalten
     *
     * GPIOx
     * Pin: Bitmaske */
    static_always_inline void io_toggleBit(GPIO_t * const GPIOx, const uint8_t Pin) 
    {
	    (void) GPIOx;
		(void) Pin;
    }	
	

    /* Logischen Wert eines Bits lesen
     * Liefert logische 1, wenn Bit gesetzt, aber nicht notwendigerweise 
     * 0x01. 
     *
     * GPIOx
     * bit: Bitmaske */
	static_always_inline uint8_t io_bitIsSet(GPIO_t * const GPIOx, const uint8_t Pin)
	{
	    (void) GPIOx;
		(void) Pin;
		return 0;
	}		



    /* Einzelne Pins eines Ports per Bitmaske als frei floatenden Eingang
     * setzen. 
     *         
     * GPIOx
     * Pin: Bitmaske */
    static_always_inline void io_setInput(GPIO_t * GPIOx, const uint8_t Pin)
	{
	    (void) GPIOx;
		(void) Pin;
	}



    /* Einzelne Pins eines Ports per Bitmaske als Push-Pull-Ausgang setzen.
     * Die Ports-Bits werden dabei nicht beeinflusst, muessen also bei 
     * Bedarf vorher oder nachher einzeln gesetzt oder geloescht werden.
     *         
     * GPIOx
     * Pin: Bitmaske */
    static_always_inline void io_setOutput(GPIO_t * GPIOx, const uint8_t Pin)
    {
       	(void) GPIOx;
		(void) Pin;
    }



    /* Einzelne Pins eines Ports per Bitmaske als Eingang mit Pull-Up
     * setzen. 
     *         
     * GPIOx
     * Pin: Bitmaske */
    static_always_inline void io_setInputPU(GPIO_t * const GPIOx, const uint8_t Pin)
    {
        (void) GPIOx;
		(void) Pin;
    }		


	/* Jede Menge Pin- und Portnamen, um originales I/O-Mapping nutzen zu 
	 * koennen */
    #define PA0 0
    #define PA1 1
    #define PA2 2        
    #define PA3 3        
    #define PA4 4        
    #define PA5 5        	
    #define PA6 6        
    #define PA7 7   	 
	 
    #define PB0 0
    #define PB1 1
    #define PB2 2        
    #define PB3 3        
    #define PB4 4        
    #define PB5 5        	
    #define PB6 6        
    #define PB7 7        

    #define PC0 0
    #define PC1 1
    #define PC2 2        
    #define PC3 3        
    #define PC4 4        
    #define PC5 5        	
    #define PC6 6        
    #define PC7 7  
    
    #define PD0 0
    #define PD1 1
    #define PD2 2        
    #define PD3 3        
    #define PD4 4        
    #define PD5 5        	
    #define PD6 6        
    #define PD7 7        
	
    extern uint8_t PORTA;    
    extern uint8_t PORTB;    
    extern uint8_t PORTC;    
    extern uint8_t PORTD;
    
#endif 
