﻿/*
 * platform.h
 *
 * Unterscheiden der unterschiedlichen Build-Plattformen
 *
 * Created: 24.08.2015 07:53:09
 *  Author: Nicolas
 */ 


#ifndef PLATFORM_H
    #define PLATFORM_H

    #ifdef __AVR__
        // AVR-GCC liefert schon ein Symbol zur Erkennung mit
        #define PLATFORM_AVR
        
        // Auf dem AVR: while(loop()) == while(1)
		static inline __attribute__((always_inline)) int loop(void) {return 1;}

    #elif defined(STM32F10x)
        // Pars pro Toto - zur Ergaenzung

        #define PLATFORM_STM32

        // Auf dem STM32: while(loop()) == while(1)
        static inline __attribute__((always_inline)) int loop(void) {return 1;}

        
    #elif defined(__MINGW64__) || ( defined(__linux__) && __linux__ == 1 )
        // PC-Plattform wird an Compiler-Symbolen erkannt (Windows oder Linux)

        #ifndef PLATFORM_PC
            #define PLATFORM_PC
        #endif
        
        // Auf dem PC: while(loop()) fragt bei jedem Schleifendurchlauf ab, ob ein 
        // unterbrechendendes Ereignis (z.B. Strg+C oder Fenster schliessen stattgefunden 
        // hat.
		extern int loop(void);
        
    #else
        #error "unknown target platform"

    #endif

#endif

