/*
 * assert.c
 *
 * Re-Implementierung des assert-Standard-Headers, um targetspezifische
 * Eigenheiten zu beruecksichtiten, insbesondere der Notwendigkeit eines
 * manuellen Refreshs der Ausgabe.
 *
 * Created: 30.04.2014
 *  Author: Nicolas
 */ 


/* Die Funktionen assert_P_dontCallMe() und error_P_dontCallMe() werden nie 
 * direkt aufgerufen, sondern nur ueber die Makros assert() und error()! */


#include <stdio.h>
#include "../src_avrcom/pgmspace.h" // PSTR, fprintf_P
#include "assert.h"

extern void abort(void);


 
/* Fehlermeldung darstellen und abort() 
 * f: Funktionsname. (__func__)
 *    __func__ ist eine builtin-Funktion und kein Makro, kann also nicht mit dem
 *    PSTR-Makro zusammen genutzt werden. Deshalb ist der Datentyp const char *
 *    und nicht constflash char *.
 * l: Zeilennummer (__LINE__)
 * e: Assertion-Text (z.B. " 0==1 ")   */
int assert_P_dontCallMe(const char *f, const int l, constflash char *e)
{
	//fprintf_P(stderr,PSTR("%s(),%d assertion failed: '%s' \a"),f,l,e);
    fprintf_P(stderr,PSTR("%s(),%d assertion failed: '"), f, l);
    fputs_P(e,stderr);
    fprintf_P(stderr,PSTR("' \a"));
	abort();
	return 0;
}


/* Fehlermeldung darstellen und abort() 
 * f: Funktionsname. (__func__)
 *    __func__ ist eine builtin-Funktion und kein Makro, kann also nicht mit dem
 *    PSTR-Makro zusammen genutzt werden. Deshalb ist der Datentyp const char *
 *    und nicht constflash char *.
 * l: Zeilennummer (__LINE__)
 * e: Fehlertext (z.B. " dumm gelaufen ")   */
void error_P_dontCallMe(const char *f, const int l, constflash char *e)
{
	//fprintf_P(stderr,PSTR("%s(),%d error: %s \a"),f,l,e);
    // Ausgabe muss fuer Flash-Strings zerpflueckt werden:
    fprintf_P(stderr,PSTR("%s(),%d error: "), f, l);
    fputs_P(e,stderr);
    fprintf_P(stderr,PSTR(" \a"));
	abort();
}

