/*
 * bitmath.h
 *
 * Einfache Funktionen fuer die Bitmasken & Co.
 *
 * Created: 02.05.2015
 *  Author: DL1DOW
 */ 

#ifndef BITMATH_H
    #define BITMATH_H
    
    #include "../src_base/datatypes.h"

    


	/* Byte strecken, indem jedes Bit verdoppelt wird, z.B.:
	 * 01001100 -> 0011000011110000
	 * Die Funktion sollte mit konstantem Parameter aufgerufen werden, damit sie
	 * statisch wegoptimiert werden kann.
	 * byte: Bitmaske
	 * return: gestreckte Bitmaske */
	static_always_inline uint16_t stretch2bitmask(const uint8_t byte)
	{
	    uint16_t out = 0;
	    out |= byte & 0x01U ?  0x0003U : 0;
	    out |= byte & 0x02U ?  0x000CU : 0;
	    out |= byte & 0x04U ?  0x0030U : 0;
	    out |= byte & 0x08U ?  0x00C0U : 0;
	    out |= byte & 0x10U ?  0x0300U : 0;
	    out |= byte & 0x20U ?  0x0C00U : 0;
	    out |= byte & 0x40U ?  0x3000U : 0;
	    out |= byte & 0x80U ?  0xC000U : 0;
	    return out;
	}




	/* Byte strecken, indem jedes Bit vervierfacht wird, z.B.:
	 * 01001100 -> 0000111100000000111111110000000000000000
	 * Die Funktion sollte mit konstantem Parameter aufgerufen werden, damit sie
	 * statisch wegoptimiert werden kann.
	 * byte: Bitmaske
	 * return: gestreckte Bitmaske */
	static_always_inline uint32_t stretch4bitmask(const uint8_t byte)
	{
	    uint32_t out = 0;
	    out |= byte & 0x01U ?  0x0000000FU : 0;
	    out |= byte & 0x02U ?  0x000000F0U : 0;
	    out |= byte & 0x04U ?  0x00000F00U : 0;
	    out |= byte & 0x08U ?  0x0000F000U : 0;
	    out |= byte & 0x10U ?  0x000F0000U : 0;
	    out |= byte & 0x20U ?  0x00F00000U : 0;
	    out |= byte & 0x40U ?  0x0F000000U : 0;
	    out |= byte & 0x80U ?  0xF0000000U : 0;
	    return out;
	}

    
#endif

