/*
 * datatypes.h
 *
 * Datentypen
 *
 * Created: 27.09.2013 
 *  Author: Nicolas
 */ 

#ifndef DATATYPES_H
    #define DATATYPES_H

    #include <stdint.h>
	#include <stdbool.h>

    /* Plattformspezifische Datentypen */
    #ifdef AVR
        // AVR-stdlib hat kein EXIT_SUCCESS/EXIT_FAILURE
        #define EXIT_SUCCESS 0
        #define EXIT_FAILURE 1
    
        typedef __int24  int_least24_t;
        typedef __uint24 uint_least24_t;
        typedef __int24  int_fast24_t;
        typedef __uint24 uint_fast24_t;
        typedef __uint24 uint24_t;
        
        #define static_always_inline static inline __attribute__((always_inline))
        #define static_inline static inline 

        // Attribute packed fuer Bitfields 
        #define STRUCT_ATTRIBUTE_MAX_PACKED __attribute__((packed))


    #elif defined STM32
        #include <stdlib.h>
    
        typedef int32_t  int_least24_t;
        typedef uint32_t uint_least24_t;
        typedef int32_t  int_fast24_t;
        typedef uint32_t uint_fast24_t;
        
        #define static_always_inline static inline __attribute__((always_inline))
        #define static_inline static inline 
    
        // Attribute packed fuer Bitfields
        #define STRUCT_ATTRIBUTE_MAX_PACKED __attribute__((packed))    
    
    #elif defined _MSC_VER  
        typedef int32_t  int_least24_t;
        typedef uint32_t uint_least24_t;
        typedef int32_t  int_fast24_t;
        typedef uint32_t uint_fast24_t;
        
        #define static_always_inline __forceinline
        #define static_inline static _inline 
		
    #elif defined __MINGW64__ || defined __linux__
        #include <stdlib.h>
        
        typedef int32_t  int_least24_t;
        typedef uint32_t uint_least24_t;
        typedef int32_t  int_fast24_t;
        typedef uint32_t uint_fast24_t;
        // MinGW kann keinen genau 24 Bit breiten Datentyp zur Verfuegung stellen
        // uint24_t muss also situationsabhaengig definiert/vermieden werden.        
        
        #define static_always_inline static inline __attribute__((always_inline))
        #define static_inline static inline 
        
        // Attribute packed fuer Bitfields:
        // __attribute__((gcc_struct)): MinGW erzeugt ohne dieses Attribut oder den 
        // Compilerschalter "-mno-ms-bitfields" ein struct mit 5 Bytes Groesse. Das ist 
        // ein Kompatibilitaets-Feature des GCC als Zugestaendnis an das Verhalten des 
        // MSVC
        #define STRUCT_ATTRIBUTE_MAX_PACKED __attribute__((packed)) __attribute__((gcc_struct))
        
    #else
        #error "No MCU defined"
        
    #endif

	/* Plattformunabhaengige Datentypen */
    typedef void (*voidFcn_t)(void);

#endif // DATATYPES_H
