/*
 * eeprom_24Cx.h
 *
 * Lesen und Schreiben eines externen EEPROMs 24C02 per I2C
 *
 * Created: 02.02.2014
 *  Author: DL1DOW
 */ 

#ifndef EEPROM_24CX_H
    #define EEPROM_24CX_H

    #define EEPROM_I2C_DEVICE I2C1
    #define EEPROM_I2C_ADDRESS 0xA0

    #define EEPROM_SIZE_BYTES 128

    enum {
        ee_ok,
        ee_invalidAddress,
        ee_noStart,
        ee_invalidSize,
		ee_error,       // interner Fehler
    };

    uint_fast8_t ee24Cx_read_byte(uint8_t addr, uint8_t *byte);
    uint_fast8_t ee24Cx_write_byte(uint8_t addr, uint8_t byte);
    uint_fast8_t ee24Cx_read_block(uint8_t *buffer,uint8_t addr,uint8_t size);
    uint_fast8_t ee24Cx_write_block(uint8_t *buffer,uint8_t addr,uint8_t size);
    uint_fast8_t ee24Cx_update_block(uint8_t *buffer,uint8_t addr,uint8_t size);

    #ifndef AVR
        /* Der AVR hat EEPROM-Routinen mit den Namen eeprom_read_byte & Co.
         * Fuer andere Plattformen springt diese Library ein.
         * Beim AVR sind die EEPROM-Adressen Zeiger (was die Einlagerung mit 
         * EEMEM erlaubt), hier sind die Adressen einfach Integer. */
        #define eeprom_read_byte    ee24Cx_read_byte
        #define eeprom_write_byte   ee24Cx_write_byte
        #define eeprom_read_block   ee24Cx_read_block
        #define eeprom_write_block  ee24Cx_write_block
        #define eeprom_update_block ee24Cx_update_block
    #endif
#endif
