/*
 * key_avr_stm32.c
 *
 * Einzeltaster entprellen und auswerten
 *
 * Refactored: 19.12.2013
 *  Author: Nicolas
 */


#include "key.h"
#include <stdbool.h>
#include "../src_avrcom/io.h"
#include "../src_misc/clock.h"
#include "../../ioMapping.h"




/* Taster initialisieren (Port) */
void key_init(void)
{
    io_setInputPU(TAST_GPIO,TAST_Pin);

	for ( uint_least8_t i = 0; i<10; i++ ) key_getstroke();
}


/* Taster abfragen und entprellen (active Low), genau eine positive Ausgabe pro
 * Tastendruck. Zum Pollen in der Hauptschleife */
static uint8_t gl_inpstat = 0x00; // Status Pin
bool key_getstroke(void)
{
	static bool oldstat = 1;
	static bool newstat = 0;
	char out;
	bool temp;

	temp = io_bitIsSet(TAST_GPIO,TAST_Pin);

	gl_inpstat = (gl_inpstat<<1) | (temp!=0);
	if (gl_inpstat == 0x00)
		newstat = 1;
	else if (gl_inpstat == 0xFF)
		newstat = 0;

	if ((newstat == 1) && (oldstat == 0))
		out = 1;
	else
		out = 0;

	oldstat = newstat;
	return out;
}


/* Dauer eines Tastendrucks abfragen (nach dem Loslassen),
 * Taster abfragen und entprellen (active Low)
 * (zum Pollen in Hauptschleife) 
 *
 * - benoetigt den laufenden Timer glu_millis mit mindestens 16 Bit 
 * - laeuft bei 1ms-Timer nach ca. 1 Minute ber 
 * 
 * return: 0 oder einmalig Dauer des letzten Tastendrucks */
uint_fast16_t key_gettime(void)
{
	static uint_fast16_t lasttime = 0;
    uint_fast16_t time = 0;    
    static char oldstat = 0x00; // Letze Ausgabe
    static char newstat = 2;    
	bool isHigh;

    // Taster abfragen
	isHigh = io_bitIsSet(TAST_GPIO, TAST_Pin);

    // Zustand Entprellen
	gl_inpstat = (gl_inpstat<<1) | (isHigh!=0);
	if (gl_inpstat == 0x00) 
    {
		newstat = 1;
	}
	else if (gl_inpstat == 0xFF) 
    {
		newstat = 0;
	}

	// Steigende Flanke
	if ((newstat == 1) && (oldstat == 0)) 
    {
		lasttime = glu_millis; // Wird abgeschnitten auf 16 Bit
    }    

	// Fallende Flanke
	if ((newstat == 0) && (oldstat == 1)) 
    {
		time = glu_millis; // Wird abgeschnitten auf 16 Bit
        time -= lasttime;  // Differenz laeuft ueber nach ca. 65 Sekunden
        
        // Damit wenigstens ein Tastendruck erkannt wird, wenn der Timer nicht 
        // laeuft
        time++;            
	}

	oldstat = newstat;
    
    return time;
}




