/*
 * Ansteuerung Grafik-LCD TG12864B
 * AVR- und STM32-spezifische Funktionen
 *
 * Created: 07.12.2013
 *  Author: Nicolas
 */ 



#include <stddef.h>
#include <stdlib.h>
#include <stdint.h>
#include "../src_avrcom/delay.h"
#include "../src_avrcom/io.h"
#include "glcd_ks0108.h"
#include "glcd_textfun.h"
#include "../../ioMapping.h"




#ifdef AVR
    // Auf dem AVR benoetigt der Datenbus die gesamte Portbreite
    #define GLCD_KS0108_DATA_OFFSET 0

#elif defined(STM32F10x)

#else
    #error "unknown MCU"
    
#endif




/* Ports auf Ausgang (GPIO-Takt muss schon vorher eingeschaltet sein!) */
void glcd_ks0108_setIo(void)
{

    // Daten-Port (8 Bit Breite)
    io_setOutput(GLCD_KS0108_DATA_GPIO, 0xFF<<GLCD_KS0108_DATA_OFFSET);

    // Steuer-Pins (muessen einzeln gesetzt werden, da evtl. an unterschiedlichen GPIOs
    io_setOutput(GLCD_KS0108_DI_GPIO, GLCD_KS0108_DI_Pin);
    io_setOutput(GLCD_KS0108_RW_GPIO, GLCD_KS0108_RW_Pin);
    io_setOutput(GLCD_KS0108_ECLK_GPIO, GLCD_KS0108_ECLK_Pin);
    io_setOutput(GLCD_KS0108_CS1_GPIO, GLCD_KS0108_CS1_Pin);
    io_setOutput(GLCD_KS0108_CS2_GPIO, GLCD_KS0108_CS2_Pin);
    io_setOutput(GLCD_KS0108_CS2_GPIO, GLCD_KS0108_CS2_Pin);
    io_setOutput(GLCD_KS0108_RESET_GPIO, GLCD_KS0108_RESET_Pin);
}






/* Sicherheitsabstand zu Power on */
void glcd_ks0108_startdelay(void)
{
    //delay_ms(50);
}


// GLCD reseten gemaess Datenblatt S. 16
void glcd_ks0108_reset(void)
{
	io_clearBit(GLCD_KS0108_RESET_GPIO, GLCD_KS0108_RESET_Pin);
	delay_us(2);
	io_setBit(GLCD_KS0108_RESET_GPIO, GLCD_KS0108_RESET_Pin);
	io_setByte(GLCD_KS0108_DATA_GPIO, 0x00, GLCD_KS0108_DATA_OFFSET);
	io_setInput(GLCD_KS0108_DATA_GPIO, 0xFF<<GLCD_KS0108_DATA_OFFSET);
	delay_us(2);

	// Warten auf Status-Flag
	while( io_bitIsSet(GLCD_KS0108_DATA_GPIO, 0x0010<<GLCD_KS0108_DATA_OFFSET) );
    
	io_setOutput(GLCD_KS0108_DATA_GPIO, 0xFF<<GLCD_KS0108_DATA_OFFSET);
}


/* Busy-Flag pollen und warten */
static void glcd_ks0108_waitWhileBusy(void)
{
	// Als Input setzen und Pull-Ups aktivieren
	io_setInput(GLCD_KS0108_DATA_GPIO, 0xFF<<GLCD_KS0108_DATA_OFFSET);
	io_setByte(GLCD_KS0108_DATA_GPIO, 0xFF,GLCD_KS0108_DATA_OFFSET);

	io_clearBit(GLCD_KS0108_DI_GPIO, GLCD_KS0108_DI_Pin);
	io_setBit(GLCD_KS0108_RW_GPIO, GLCD_KS0108_RW_Pin);

	// poll busy flag
	do {
		io_clearBit(GLCD_KS0108_ECLK_GPIO, GLCD_KS0108_ECLK_Pin);
		delay_us(0.450);
		io_setBit(GLCD_KS0108_ECLK_GPIO, GLCD_KS0108_ECLK_Pin);
		delay_us(0.450);

	} while( io_bitIsSet(GLCD_KS0108_DATA_GPIO, 0x0080<<GLCD_KS0108_DATA_OFFSET) );

	io_clearBit(GLCD_KS0108_ECLK_GPIO, GLCD_KS0108_ECLK_Pin);
	io_clearBit(GLCD_KS0108_RW_GPIO, GLCD_KS0108_RW_Pin);

	io_setOutput(GLCD_KS0108_DATA_GPIO, 0xFF<<GLCD_KS0108_DATA_OFFSET);
}


/* ************************************************************************************ *
 *     MCU-unabhaengige, extern verwendete Funktionen                                   *
 * ************************************************************************************ */


/* Daten von GLCD auslegen - nie getestet!                                    */
uint8_t glcd_ks0108_readdat(void)
{
	uint8_t output;

	io_setInput(GLCD_KS0108_DATA_GPIO, 0xFF<<GLCD_KS0108_DATA_OFFSET);
	glcd_ks0108_waitWhileBusy();

	io_setBit(GLCD_KS0108_RW_GPIO, GLCD_KS0108_RW_Pin);
	io_setBit(GLCD_KS0108_DI_GPIO, GLCD_KS0108_DI_Pin);

	//delay(1);
	io_setBit(GLCD_KS0108_ECLK_GPIO, GLCD_KS0108_ECLK_Pin);
	//_delay_ms(1);
	output = io_getByte(GLCD_KS0108_DATA_GPIO, GLCD_KS0108_DATA_OFFSET);
	io_clearBit(GLCD_KS0108_ECLK_GPIO, GLCD_KS0108_ECLK_Pin);
	io_clearBit(GLCD_KS0108_RW_GPIO, GLCD_KS0108_RW_Pin);

	while( io_bitIsSet(GLCD_KS0108_DATA_GPIO, 0x0080<<GLCD_KS0108_DATA_OFFSET) ) 
    {
        //busy abwarten
	}; 
    
	io_clearBit(GLCD_KS0108_DI_GPIO, GLCD_KS0108_DI_Pin);

	return output;
} 


/* Zeichen oder Kommando an Display senden
 * RS : 0=Instruction 1=data                                                  */
void glcd_ks0108_putsth(char value,uint8_t RS)
{
	// Neuimplementierung write-Zyklus gemaess Datenblatt S. 14

    #ifdef GLCD_KS0108_BUSYTIME
        delay_us(GLCD_KS0108_BUSYTIME);
    #else
        glcd_ks0108_waitWhileBusy();
    #endif

	io_clearBit(GLCD_KS0108_ECLK_GPIO, GLCD_KS0108_ECLK_Pin); // Enable Low
	io_clearBit(GLCD_KS0108_RW_GPIO, GLCD_KS0108_RW_Pin);   // Write
    
	if(RS)
		io_setBit(GLCD_KS0108_DI_GPIO, GLCD_KS0108_DI_Pin);   // Data
	else
		io_clearBit(GLCD_KS0108_DI_GPIO, GLCD_KS0108_DI_Pin); // Instruction
        
	delay_us(0.450); // max(t_ASU,t_t_WL)
	io_setBit(GLCD_KS0108_ECLK_GPIO, GLCD_KS0108_ECLK_Pin);
	io_setByte(GLCD_KS0108_DATA_GPIO, value, GLCD_KS0108_DATA_OFFSET);
	delay_us(0.450);  // max(t_DSU, t_WH)
	io_clearBit(GLCD_KS0108_ECLK_GPIO, GLCD_KS0108_ECLK_Pin); // Enable Low
}


/* Displayseite waehlen (L,R)  */
void glcd_ks0108_area(uint8_t LR)
{
	if(!LR) 
    {  // Left/Right
		io_setBit(GLCD_KS0108_CS1_GPIO,GLCD_KS0108_CS1_Pin);
		io_clearBit(GLCD_KS0108_CS2_GPIO,GLCD_KS0108_CS2_Pin);
	} 
    else 
    {
		io_clearBit(GLCD_KS0108_CS1_GPIO,GLCD_KS0108_CS1_Pin);
		io_setBit(GLCD_KS0108_CS2_GPIO,GLCD_KS0108_CS2_Pin);
	}
}
