/*
 * glcd_textfun.h
 *
 * Textfunktionen fr Grafikdisplays mit nur einem Display-
 * controller, d.h. durchgehend adressiertem SRAM
 * Die Darstellung von Text funktioniert bislang nur im
 * logischen Zeilenraster des LCDs (d.h. Schrifthhe max.
 * 7 Pixel und nur im 8-Pixel-Raster positionierbar).
 
 * Allein eine 5x7-Schriftart einzubetten kostet 500 bytes.
 *
 * Unterstuetzte Excape-Sequenzen:
 * \f Seitenvorschub (glcd_clear)
 * \a, EOM Alarm (sofort darstellen, glcd_drawnow)
 * \b Backspace
 * \t Tab
 * \n Newline
 * \e Escape (weiterer Befehl, z.B. andere Schriftart)
 *
 * Autor: Nicolas
 */


#ifndef GLCD_TEXTFUN_H
    #define GLCD_TEXTFUN_H


    #include "../../main.h"
    #include "./glcd_generic.h"
    #include "../src_avrcom/pgmspace.h"

    #define N_TEXTBUF 81
    char glstr_Buf[N_TEXTBUF];        // String fuer Textoutput


    #define DEFAULTFONT Font57Lookup
    #define TABWIDTH 32U
    #define LINEWIDTH 128U

    #define EOM "\a"  // Display-Update


    // Einzelnes Zeichen ausgeben
    void glcd_putchar(char c);
    
    // Strings aus RAM auf LCD ausgeben                               
    void glcd_putstr(char *zeichenkette);                
    
    // Strings aus Flash auf LCD ausgeben
    void glcd_putstr_P(constflash char *zeichenkette);
    
    // Alle Zeichen des aktuellen Fonts ausgeben
    void glcd_test_chars(void);                          


    // Externe Funktionsprototypen (muessen von Displayroutine bereitgestellt werden
    void glcd_putdat(uint8_t value);
    void glcd_init(void);
    void glcd_clear(void);
    void glcd_setpos(uint16_t Pos);
    uint16_t glcd_getpos(void);
    void glcd_drawnow(void);




#endif //GLCD_TEXTFUN_H


