/*
 * i2c_simple.h
 *
 * Einfache Variante von I2C-Routinen
 * als Fingeruebung zur Inbetriebnahme
 *
 * Auf dem AVR ist die "simple"-Implementierung voellig in Ordnung
 *
* Fuer die Timeouts wird erwartet, dass der Timeout-Timer (setTimeout und
 * pollTimeout) laeuft (sonst wird z.B. der Scan steckenbleiben)
 *
 * 22.12.13 DL1DOW
 */


#ifndef I2C_SIMPLE_H
    #define I2C_SIMPLE_H

    #include "i2c_generic.h"

    #ifdef AVR
    	// I2C-Konstanten wie beim STM32 fuer den AVR definieren. Da der AVR nur
    	// einen I2C hat wird diese Einstellung ignoriert
    	#define  I2C_Direction_Transmitter      ((uint8_t)0x00)
    	#define  I2C_Direction_Receiver         ((uint8_t)0x01)
    	#define  I2C1 0x00
    	#define  I2C2 0x00

    #elif defined(STM32F10x)
    	#include "stm32f10x_gpio.h"
    	#include "stm32f10x_rcc.h"
    	#include "stm32f10x_i2c.h"

    #elif defined(STM32F4xx)
    	#include "stm32f4xx_gpio.h"
    	#warning "not implented and tested for STM32F4xx"

   	#elif defined _MSC_VER || defined __MINGW64__ || \
		( defined(__linux__) && __linux__ == 1 )
    	#define  I2C_Direction_Transmitter      ((uint8_t)0x00)
    	#define  I2C_Direction_Receiver         ((uint8_t)0x01)
    	#define  I2C1 0x00
    	#define  I2C2 0x00
    	
    #else
    	#error "unknown MCU"
    #endif


    void I2C_polltimeout(void); // must be called in an isr about every 1ms

    uint_fast8_t I2C_init(I2C_TypeDef* I2Cx);  // definiert in i2c_generic
    uint_fast8_t I2C_reset(I2C_TypeDef* I2Cx); // definiert in i2c_generic

    // High-level Sende- und Empfangsfunktionen, die bevorzugte Methode
    uint_fast8_t
    I2C_Sendbuffer(I2C_TypeDef *I2Cx, uint8_t slaveaddr,
  	   uint8_t* pBuffer, uint8_t nBuffer, enum stopbit_e stopbit);

    uint_fast8_t
    I2C_Receivebuffer(I2C_TypeDef *I2Cx, uint8_t slaveaddr,
  	  	uint8_t* pBuffer, uint8_t nBuffer);


#endif //I2C_SIMPLE_H
