/*
 * menu.h
 *
 * - Darstellung eines Menues
 * - Darstellung von "Popup-Fenstern"
 *
 * Created: 10.07.2013 18:15:08
 *  Author: Nicolas
 */


#ifndef MENU_H
    #define MENU_H

    #include "../src_avrcom/pgmspace.h"
    #include "../src_base/datatypes.h"
  

	enum datatype_e 
    {
        nothing,
		voidFcn,
		uint8,
		uint16,
		int8,
		int16
	};


    // Menue-Eintraege: Zu startende Funktionen, zu aendernde Variablen
    __extension__ typedef struct 
    {
	    constflash char *menutext;  // Menueeintrag/Funktionsname
	    enum datatype_e datatype;   // Datentyp des Zeigers "pointer"
	                                // Bei function pointer wird die Funktion
								    // gestartet, bei Zeigern auf normale Daten
								    // der Datentyp im Menue bearbeitet.
        union 
        {
            voidFcn_t pvoidFcn;     // Funktionszeiger
            uint8_t  *puint8;       // Zeiger auf Variablentypen
            uint16_t *puint16;
            int8_t   *pint8;
            int16_t  *pint16;
            void     *pvoid;
        };
	    const int16_t varMin;       // Untere Grenze numerische Variable
	    const int16_t varMax;       // Oberere Grenze numerische Variable
        const int8_t  varInc;       // Inkrement bei einer Rastung
    } menudata_t;                   // Struct wird terminiert mit einer Zeile,
                                    // wo in .menutext ein Nulltext
                                    // ({'\0'}) steht.

    // Menue-Konfiguration                              
    typedef struct 
    {
        uint8_t headheight;           // Hoehe der Kopfzeile
        constflash char *headtext;    // Kopfzeile (Text)
        voidFcn_t pheadfkt;           // Funktionszeiger auf Funktion, die bei
                                      // jedem Schleifendurchlauf aufgerufen
                                      // wird, z.B. zum speziellen Darstellen
                                      // der Kopfzeile.
    } menuconfig_t;


    typedef constflash char * const usermessageitem_t;
                      
    typedef struct 
    {
        constflash char *text;           // Text (wird von "printf" interpretiert), eine
                                         // einzelne int-Variable erlaubt, typischerweise
                                         // ein Fehlercode
        int errcode;                     // Der Fehlercode                                 
        constflash unsigned char *icon;  //
        constflash char * const * items; //
        
        uint8_t headheight;
    } usermessage_t;
                      
  

  
    // Menue kann zu startende Funktionen, anzuzeigenden Text und zu aendernde 
    // Variablen erhalten. Meist ist der Rueckgabewert nebensaechlich
    uint_fast8_t menu_standard(constflash menudata_t menudata[],
                        constflash menuconfig_t *menuconfig);

    // Usermessage ist ein vereinfachtes Menue mit N anzuzeigenden Texten und
    // einem Rueckgabewert, der anzeigt, welcher Text zum Beenden aufgerufen
    // wurde.
    uint_fast8_t usermessage(usermessage_t *message);


#endif /* MENU_H */

