/*
 * menu_tests
 *
 * Men fr Hardware- und Funktionstests
 *
 * Created: 9.02.2014
 *  Author: DL1DOW
 */


#include "../src_avrcom/pgmspace.h"
#include "../src_glcd/glcd_ks0108.h"
#include "../src_glcd/glcd_textfun.h"
#include "../src_glcd/glcd_test.h"
#include "../src_encoder/encoder_test.h"
#include "../src_encoder/key_test.h"
#include "../src_avrcom/delay_test.h"
#include "../src_i2c/i2c_test.h"
#include "../src_eeprom/eeprom_test.h"
#include "../src_i2c/i2c_lm75.h"
#include "../src_pwm/pwm_test.h"
#include "../src_adc/adc_test.h"
#include "../src_motor/tmc222_simple_test.h"
#include "../src_misc/memdump.h"   // sramdump(), flashdump(), eepromdump()
#include "../src_misc/crc_test.h"
#include "../src_edm/edm_adc_test.h"
#include "../src_edm/edm_feed_test.h"
#include "../src_ui/ui_menu_eeprom.h"
#include "../src_ui/ui_infoscreen.h"
#include "../src_ui/bitmaps.h"
#include "../src_misc/clock_test.h"
#include "../src_edm/edm_screen_test.h"
#include "../src_edm/edm_userinput_test.h"

#include "../src_ui/locale.h"
#include "menu.h"


void menu_msgbox_test(void);








constflash char MT_SCAN_I2C[]       = "Scan I2C bus";
constflash char MT_RESET_I2C[]      = "Reset I2C bus";


constflash char MT_TEST_USERINPUT[] = "Test: EDM-Input";
constflash char MT_TEST_SCREEN[]    = "Test: EDM-Screen";
constflash char MT_EDM_FEED_TEST[]  = "Test: EDM Verfahren";
constflash char MT_EDM_ADC_TEST[]   = "Test: EDM-ADC";
constflash char MT_TEST_TMC222[]    = "Test: TMC222";
constflash char MT_TEST_PWM[]       = "Test: PWM";
constflash char MT_LM75_TEST[]      = "Test: LM75";
constflash char MT_EEPROMTEST_R[]   = "Test: EEPROM (R)";
constflash char MT_EEPROMTEST_RB[]  = "Test: EEPROM (RB)";
constflash char MT_EEPROMTEST_W[]   = "Test: EEPROM (W)";
constflash char MT_EEPROMTEST_WB[]  = "Test: EEPROM (WB)";






/* Menue Hilfsfunktionen
 *
 * Hilfsfunktionen sind Funktionen, die spaeter (an anderer Stelle) von der
 * Firmware unveraendert gebraucht werden. */
static constflash menudata_t Menu_hlpfcn_dat[] =
{
    {MT_RETURN        ,voidFcn,{.pvoid=NULL}                   ,0,0,0},
//  {MT_PREFSMENU     ,voidFcn,{.pvoidFcn=edm_prefsmenu}       ,0,0,0},
//	{MT_EDM_MENU      ,voidFcn,{.pvoidFcn=edm_menu}            ,0,0,0},
    {MT_INFOSCREEN    ,voidFcn,{.pvoidFcn=infoscreen}          ,0,0,0},
    {MT_EDM_WRITE_FCN ,voidFcn,{.pvoidFcn=ui_write_eeprom} ,0,0,0},
    {MT_EDM_READ_FCN  ,voidFcn,{.pvoidFcn=ui_read_eeprom}  ,0,0,0},
    {MT_SRAMDUMP      ,voidFcn,{.pvoidFcn=sramdump}            ,0,0,0},
    {MT_FLASHDUMP     ,voidFcn,{.pvoidFcn=flashdump}           ,0,0,0},
    {MT_EEPROMDUMP    ,voidFcn,{.pvoidFcn=eepromdump}          ,0,0,0},
    {MT_SCAN_I2C      ,voidFcn,{.pvoidFcn=i2c_scan}            ,0,0,0},
    {MT_RESET_I2C     ,voidFcn,{.pvoidFcn=i2c_reset_all}       ,0,0,0},
    {MT_SPLASHSCREEN  ,voidFcn,{.pvoidFcn=splashscreen_test}   ,0,0,0},
    // Nullzeile:
    {MT_NULL          ,nothing,{.pvoid=NULL}                   ,0,0,0},
};


/* Menue Testfunktionen
 *
 * Testfunktionen dienen allein dem Test einzelner Softwarekomponenten und
 * werden im eigentlichen Programmablauf spaeter nicht mehr benoetigt. */
static constflash menudata_t Menu_tstfcn_dat[] =
{
    {MT_RETURN        ,voidFcn,{.pvoid=NULL}                        ,0,0,0},
    {MT_TEST_USERINPUT,voidFcn,{.pvoidFcn=edm_userinput_test_visual},0,0,0},
    {MT_TEST_SCREEN   ,voidFcn,{.pvoidFcn=edm_screen_test_visual}   ,0,0,0},
    {MT_EDM_FEED_TEST ,voidFcn,{.pvoidFcn=edm_test_manualfeed}      ,0,0,0},
    {MT_EDM_ADC_TEST  ,voidFcn,{.pvoidFcn=edm_test_adc}             ,0,0,0},
    {MT_MESSAGE_TEST  ,voidFcn,{.pvoidFcn=menu_msgbox_test}         ,0,0,0},
    {MT_TEST_TIMEOUT  ,voidFcn,{.pvoidFcn=timeout_test_visual}      ,0,0,0},
    {MT_TEST_CRC16    ,voidFcn,{.pvoidFcn=crc16_test}               ,0,0,0},
    {MT_TEST_TMC222   ,voidFcn,{.pvoidFcn=tmc222_simple_test}       ,0,0,0},
    {MT_TEST_ADC      ,voidFcn,{.pvoidFcn=adc_test}                 ,0,0,0},
    {MT_TEST_PWM      ,voidFcn,{.pvoidFcn=pwm_test}                 ,0,0,0},
    {MT_TEST_ENCODER  ,voidFcn,{.pvoidFcn=encoder_test}             ,0,0,0},
    {MT_TEST_KEY      ,voidFcn,{.pvoidFcn=key_test}                 ,0,0,0},
    {MT_TEST_TEXT     ,voidFcn,{.pvoidFcn=glcd_texttest}            ,0,0,0},
    {MT_TEST_DELAY    ,voidFcn,{.pvoidFcn=delay_test}               ,0,0,0},
    {MT_LM75_TEST     ,voidFcn,{.pvoidFcn=lm75_test}                ,0,0,0},
    {MT_EEPROMTEST_R  ,voidFcn,{.pvoidFcn=eeprom_test_read}         ,0,0,0},
    {MT_EEPROMTEST_RB ,voidFcn,{.pvoidFcn=eeprom_test_read_block}   ,0,0,0},
    {MT_EEPROMTEST_W  ,voidFcn,{.pvoidFcn=eeprom_test_write}        ,0,0,0},
    {MT_EEPROMTEST_WB ,voidFcn,{.pvoidFcn=eeprom_test_write_block}  ,0,0,0},
    // Nullzeile:                                                     
    {MT_NULL          ,nothing,{.pvoid=NULL}                        ,0,0,0},
};



/* Darstellung Kopfzeile */
static void head_menu_hlpfcn(void)
{
    glcd_setpos(0);
  
    uint8_t i = 0;
    do 
    {
        glcd_putdat(0xFF);
    } while(++i);

    glcd_setpos(144);
    glcd_putstr_P(PSTR("\033b\033v\033iHilfsfunktionen\033i\033n\033f"));
}


/* Darstellung Kopfzeile */
static void head_menu_tstfcn(void)
{
    glcd_setpos(0);

    uint8_t i = 0;
    do 
    {
        glcd_putdat(0xFF);
    } while(++i);

    glcd_setpos(144);
    glcd_putstr_P(PSTR("\033b\033v\033iTestfunktionen\033i\033n\033f"));
}



static constflash menuconfig_t Menu_hlpfcn_config = 
{
    2, // headheight
    NULL, // headtext
    head_menu_hlpfcn //pheadfkt
};

static constflash menuconfig_t Menu_tstfcn_config = 
{
    2, // headheight
    NULL, // headtext
    head_menu_tstfcn //pheadfkt
};




/* Menue mit Hilfsfunktionen */
void menu_hlpfcn(void)
{
    menu_standard(Menu_hlpfcn_dat,&Menu_hlpfcn_config);
}

/* Menue mit vielen Tests */
void menu_tstfcn(void)
{
    menu_standard(Menu_tstfcn_dat,&Menu_tstfcn_config);
}



/* Test von Auswahlfenstern ("Popup") */
void menu_msgbox_test(void)
{
    uint16_t status;
    usermessageitem_t items[]  = {MT_YES,MT_NO,MT_CANCEL,MT_NULL};   
    usermessage_t message;

    message.text = PSTR("   Fehlermeldung\n   h=%u");
    message.items = items;
    message.headheight = 3;
    message.icon = icon_error;
    message.errcode = 100;
  
    status = usermessage(&message);
  
    printf_P(PSTR("n = %u\a"),status);
}   
