/*
 * clock.c
 *
 * Einfache Uhrenfunktionen und globaler Timer
 *
 * Created: 25.10.2013
 *  Author: Nicolas
*/


#include "clock.h"
#include <stddef.h>
#include "../src_base/assert.h"
#include "../../main.h" //	#define CLOCK_TICKS_PER_SECOND 1000




/* Die Variable glu_millis wird in einer Hauptschleife jede Millisekunde
 * inkrementiert  und steht allen anderen Funktionen fuer Timer zur Verfuegung
 * (insbesondere fuer Timeouts). Laeuft alle 50 Tage ueber. */
volatile uint_least24_t glu_millis = 0; // Millisekundentimer

#ifndef CLOCK_TICKS_PER_SECOND
    #error "Symbol CLOCK_TICKS_PER_SECOND must be defined in main.h"
#endif


/* Einfache Uhr zum hochzaehlen. Hauptsaechlich als sichtbarer Heartbeat */
void clock_timer(void) 
{
	if (gls_time.tick==0) 
    {
		gls_time.tick = CLOCK_TICKS_PER_SECOND;
		gls_time.s++;
		if (gls_time.s==60) 
        {
			gls_time.s = 0;
			gls_time.m++;
			if (gls_time.m==60) 
            {
				gls_time.m = 0;
				gls_time.h++;
				if (gls_time.h==24) 
                {
					gls_time.h = 0;
				}
			}
		}
	}
	gls_time.tick--;
}


void clock_reset(void)
{
	gls_time.h = 0;
	gls_time.m = 0;
	gls_time.s = 0;
	gls_time.tick = 0;
}



/* Timeout-Funktionen (auf Basis des Millisekunden-Timers)
 *
 * Verwendung:
 *
 * timeoutstor_t t;
 * setTimeout(&t,10);
 * while(!pollTimeout(&(t)) {
 *     ...
 * }
 */
void setTimeout(timeoutstor_t *storage, uint_least24_t value )
{
	if (storage==NULL) 
	{
		error("Null pointer given as storage ");
	}
	else 
	{
		*storage = glu_millis + value;
	}
}



/* Timeout pollen 
 * return: 1: Timeout abgelaufen, 0: Timeout noch nicht abgelaufen */
bool pollTimeout(timeoutstor_t *storage)
{
	int_fast24_t diff;

	// Fuer die korrekte, vorzeichenrichtige Differenzbildung mit Ueberlauf
	// muessen die Datentypen alle von der gleichen Groesse sein
	static_assert(sizeof(*storage)==sizeof(glu_millis),"invalid integer types");
	static_assert(sizeof(*storage)==sizeof(diff),"invalid integer types");

    if (storage==NULL)
    {
        error("Null pointer given as storage ");
        return 0; // nur wegen warning
    }
    else
    {        
	    diff = (int_fast24_t) (*storage - glu_millis);
	    return (diff < 0);
    }    
}


int_fast24_t get_timediff(timeoutstor_t *storage)
{
	int_fast24_t diff;

	// Fuer die korrekte, vorzeichenrichtige Differenzbildung mit Ueberlauf
	// muessen die Datentypen alle von der gleichen Groesse sein
	static_assert(sizeof(*storage)==sizeof(glu_millis),"invalid integer types");
	static_assert(sizeof(*storage)==sizeof(diff),"invalid integer types");

	diff = (int_fast24_t) (glu_millis - *storage);
	return diff;
}
