/*
 * crc.c
 *
 * Berechnung eines:
 *  - CRC16 mit dem Generatorpolynom 0xA001
 *  - CRC4 mit dem Generatorpolynom 0xD 
 *
 * Created: 08.03.2014
 *  Author: Nicolas
 */ 

#include <stdint.h>
#ifdef AVR
    #include <util/crc16.h>
#endif  



/* Berechnung CRC16 mit Generatorpolynom 0xA001
 *
 * Die  AVR-Libc hat eine ASM-Implementierung eingebaut; fuer andere Plattformen wird 
 * die Implementierung aus der Doku der AVR-libs verwendet */
#ifdef AVR

    uint16_t crc16_update(uint16_t crc, uint8_t a)
    {
    	return _crc16_update(crc,a);
    }
  
#else

    uint16_t crc16_update(uint16_t crc, uint8_t a)
    {
    	uint_fast8_t i;

    	crc ^= a;
    	for (i = 0; i < 8; ++i)
    	{
    		if (crc & 1)
    			crc = (crc >> 1) ^ 0xA001;
    		else
    			crc = (crc >> 1);
    	}

    	return crc;
    }
  
#endif


/* Berechnung eines ganzen Blocks */
uint16_t crc16_calc(uint8_t *buffer,uint8_t buffersize)
{
  	uint16_t crc = 0xFFFF;
  	uint8_t i;
  	for(i=0;i<buffersize;i++) 
    {
  		crc = crc16_update(crc,*buffer++);
  	}
  	return crc;
}



/* Berechnung CRC4 mit Generatorpolynom 0xD (CRC-CCIT) analog der Implementierung aus der 
 * AVR-libc-Dokumentation 
 * crc: alte Pruefsumme
 * a: Neuer Wert (4 Bit)
 * return: neue Pruefsumme  */
uint8_t crc4_update(uint8_t crc, uint8_t a)
{
    uint_fast8_t i;
    a &= 0xF; 
    
    crc ^= a;
    for (i = 0; i < 4; ++i)
    {
        if (crc & 1)
            crc = (crc >> 1) ^ 0xD;
        else
            crc = (crc >> 1);
    }
    
    return crc;
}



/* CRC eines ganzen Blocks. Es werden von jedem Byte nur 4 Bit genutzt. 
 * Achtung: Noch nie getestet */
uint8_t crc4_calc(uint8_t *buffer, uint8_t nNibble)
{
    uint8_t crc = 0xF;
    uint8_t i;
    for(i=0;i<nNibble;i++)
    {
        crc = crc4_update(crc,*buffer++);
    }
    return crc;
}



/* CRC einer "grossen" Variable mit 32 Bit. Es werden die hinteren nNibble genutzt. */
uint8_t crc4_calc32(uint32_t inp, uint8_t nNibble)
{
    uint8_t crc = 0xF;
    uint8_t i;
    for(i=0;i<nNibble;i++)
    {
        crc = crc4_update(crc,(inp >> (4*nNibble-4)) & 0xF);
        inp<<=4;
    }
    return crc;
}
