/*
 * memdump.c
 *
 * Speicherinhalt (Flash, EEPROM, SRAM) auf dem Grafik-LCD darstellen. Jede
 * Spalte ist ein Byte, jedes Pixel ein Bit
 *
 * MCU-unabhaengige Funktionen 
 *
 * Created: 13.07.2013 11:41:19
 *  Author: Nicolas
 */

#include <stddef.h>
#include "../src_base/intmath.h"
#include "../src_glcd/glcd_generic.h"
#include "../src_glcd/glcd_textfun.h"
#include "../src_encoder/key.h"
#include "../src_encoder/encoder_panasonic.h"
#include "../src_avrcom/pgmspace.h"
#include <stdio.h>

typedef char (*readFcn_t) (size_t);


/* Allgemeine Funktion zur Darstellung des Speicherbereichs */
void memdump(readFcn_t readfcn,uintptr_t memend)
{
	
	uint8_t nPages = (memend -1)/((uint16_t) GLCD_END) + 1;
	uint8_t page = 0;
	uint8_t isChanged = 1;
	uintptr_t startaddr = 0;
	uintptr_t endaddr = 0;
    int_fast8_t diff = 0;

	// Drehgeber uebernehmen und nullen
	encoder_get_diff();
	
	while(1)	
    {
		// Darstellung SRAM-/EEPROM-/Flash-Belegung
		if (isChanged) 
        {
			glcd_clear();
			glcd_setpos(0);
			#ifdef GLCD_ABPOS_WORKAROUND
			    glcd_setpos(0);
			#endif
			
			startaddr = ((uintptr_t) GLCD_END)*page;
			endaddr = startaddr + GLCD_END;
			if (endaddr > (uintptr_t) memend) endaddr = (uintptr_t) memend;
			
			for (uintptr_t i = startaddr; i <= endaddr; i++) 
            {
				glcd_putdat(readfcn(i));
			}
		}

		// Funktion auf Knopfdruck abbrechen
		if(key_getstroke()) return;

		// Drehgeber auswerten (nur Aenderungen)
		diff = encoder_get_diff();
        isChanged = !(diff == 0);
		page = limadd_uint8(page, diff, 0U, nPages-1);

        glcd_draw();
	}
}

