/*
 * memdump.c
 *
 * Speicherinhalt (Flash, EEPROM, SRAM) auf dem Grafik-LCD darstellen. Jede Spalte ist 
 * ein Byte, jedes Pixel ein Bit
 *
 * AVR-spezifische Funktionen 
 *
 * Created: 13.07.2013 11:41:19
 *  Author: Nicolas
 */


#include <avr/io.h>       // RAMEND
#include <avr/pgmspace.h> // Flash-Funktionen
#include <avr/eeprom.h>


typedef char (*readFcn_t) (size_t);
extern void memdump(readFcn_t readfcn,size_t memend);



/* Menuehilfsfunktionen: Darstellung des SRAM-Inhalts als Pixelmuster
 * angelehnt an Artikel:
 * http://www.rn-wissen.de/index.php/Speicherverbrauch_bestimmen_mit_avr-gcc
 * (abgerufen am 1.07.2013)
 */


/* Kompletten SRAM, so weit erlaubt, mit 0xAA fuellen, um den vom Programm
 * angetasteten Bereich sichtbar zu machen (komplett aus Artikel uebernommen) */
#define MASK 0xaa

// !!! Never call this function, it is part of .init-Code
void __attribute__ ((naked, used, section(".init3"))) init_mem (void);
void init_mem (void)
{
	__asm volatile (
	"ldi r30, lo8 (__heap_start)"  "\n\t"
	"ldi r31, hi8 (__heap_start)"  "\n\t"
	"ldi r24, %0"                  "\n\t"
	"ldi r25, hi8 (%1)"            "\n"
	"0:"                           "\n\t"
	"st  Z+,  r24"                 "\n\t"
	"cpi r30, lo8 (%1)"            "\n\t"
	"cpc r31, r25"                 "\n\t"
	"brlo 0b"
	:
	: "i" (MASK), "i" (RAMEND+1)
	);
}



/* Darstellung der Speicherbelegung (SRAM) des ATmega32 als Pixelmuster auf dem GLCD.
 * Display kann nur 1k darstellen, deshalb durchscrollen mit Drehgeber. */

/* Hilfsfunktion: Byte aus SRAM lesen */
char readsram(size_t i)
{
    /* i kann auch 0 sein- der tapfere kleine AVR zeigt auch *(char *)0 an. */
	return *(char *) i;
}

/* Darstellung Speicherbelegung */
void sramdump(void)
{
	readFcn_t readfcn = readsram;
	memdump(readfcn,(size_t) RAMEND);
}


/* Darstellung der Speicherbelegung (Flash) des ATmega32 als Pixelmuster auf dem GLCD.
 * Display kann nur 1k darstellen, deshalb durchscrollen mit Drehgeber */

/* Hilfsfunktion: Byte aus Flash lesen */
char readflash(size_t i)
{
	return pgm_read_byte(i);
}

/* Darstellung Speicherbelegung */
void flashdump(void)
{
	readFcn_t readfcn = readflash;
	memdump(readfcn,(size_t) FLASHEND);
}


/* Darstellung der Speicherbelegung (EEPROM) des ATmega32 als Pixelmuster auf dem GLCD.
 * Display kann nur 1k darstellen, deshalb durchscrollen mit Drehgeber */

/* Hilfsfunktion: Byte aus EEPROM lesen */
char readeeprom(size_t i)
{
	return eeprom_read_byte((uint8_t *) i);
}

/* Darstellung Speicherbelegung */
void eepromdump(void)
{
	readFcn_t readfcn = readeeprom;
	memdump(readfcn,(size_t) E2END);
}
