/*
 * memdump.c
 *
 * Speicherinhalt (Flash, EEPROM, SRAM) auf dem Grafik-LCD darstellen. Jede
 * Spalte ist ein Byte, jedes Pixel ein Bit
 *
 * Mockup - es werden komplett sinnlose Daten zurueckgegeben.
 *
 * Created: 13.07.2013 11:41:19
 *  Author: Nicolas
 */


#include <stdint.h>
#include <stddef.h>
#include "../src_eeprom/eeprom_24Cx.h"


/* Device-Spezifikationen */
/* Auf dem PC duerfen keine wilden Speicheradressen angezeigt werden,
 * sonst Schutzverletzung */
#define FLASH_OFFSET 0x00
#define FLASH_END    1

#define SRAM_OFFSET  1024
#define SRAM_END     20



typedef char (*readFcn_t) (uintptr_t);
extern void memdump(readFcn_t readfcn,uintptr_t memend);





/* Darstellung der Speicherbelegung (SRAM-Bereich) des als Pixelmuster auf dem
 * GLCD Display kann nur 1k darstellen, deshalb durchscrollen mit Drehgeber */
char readsram(uintptr_t i)
{
	uintptr_t address = i + SRAM_OFFSET;
	return *(char *) address;
}

void sramdump()
{
	readFcn_t readfcn = readsram;
	memdump(readfcn, (uintptr_t) SRAM_END);
}


/* Darstellung der Speicherbelegung (Flash) als Pixelmuster auf dem GLCD
 * Display kann nur 1k darstellen, deshalb durchscrollen mit Drehgeber */
char readflash(uintptr_t i)
{
	uintptr_t address = i + FLASH_OFFSET;
	return *(char *) address;
}

void flashdump()
{
	readFcn_t readfcn = readflash;
	memdump(readfcn, (uintptr_t) FLASH_END);
}


/* Darstellung der Speicherbelegung (EEPROM) des externen EEPROMs.
 * EEPROM-Inhalt passt in eine Displayzeile */
char readeeprom(uintptr_t i)
{
    uint8_t byte;
    eeprom_read_byte( (uint8_t) i,&byte);
    return (char) byte;
}

void eepromdump()
{
    readFcn_t readfcn = readeeprom;
    memdump(readfcn,(uintptr_t) EEPROM_SIZE_BYTES);
}
