/*
 * mock_glcd.c
 *
 * Einfache Emulation des Displayverhaltens ueber SDL2
 * Die Emulation beschraenkt sich darauf, den Displaypuffer grafiscsh
 * darzustellen. Das Timing und die tatsaechliche Kommunikation mit dem GLCD
 * wird nicht beruecksichtigt. Ausserdem funktioniert nur der Modus mit
 * DISPLAYBUFFER.
 *
 * Created: 19.04.2015
 *  Author: DL1DOW
*/



#include <stdint.h>
#include <SDL2/SDL.h>
#include "mock_glcd.h"
#include "../src_avrcom/platform.h"



/* Der Stub ist nur fuer den PC und wurde nicht mit allen Compilern getestet */
#ifndef PLATFORM_PC
     #error "This stub is only for PC platform"
#endif




/* Groesse des emulieren Displays */
#define GLCD_DISPWIDTH  128
#define GLCD_DISPHEIGHT 64

/* Skalierung der Bildschirmdarstellung:
 * 1U : Screenshots fuer Doku
 * 2U : Gut fuer Bildschirmdarstellung */
#define SCREEN_SCALE 2U


/* Der Framebuffer fuer das echte GLCD wird hier missbraucht */
extern uint8_t gl_framebuffer[1024];

/* Variablen fuer Display-Emulation */
SDL_Window   * pWindow;
SDL_Renderer * pRenderer;
SDL_Texture * pTexture;

#define N_BYTE_PIXELBUFFER (GLCD_DISPWIDTH*GLCD_DISPHEIGHT*sizeof(uint32_t))
uint32_t pixelbuffer[N_BYTE_PIXELBUFFER];




/* Grafik-Teil des Mockups mit SDL initialisieren */
void mock_glcd_init(void)
{
    SDL_Init(SDL_INIT_VIDEO);

    pWindow = SDL_CreateWindow("Virtual FTM-Control",
            SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED,
            SCREEN_SCALE*GLCD_DISPWIDTH,
            SCREEN_SCALE*GLCD_DISPHEIGHT,
            0);

    pRenderer = SDL_CreateRenderer(pWindow, -1, 0);

    pTexture = SDL_CreateTexture(pRenderer,
            SDL_PIXELFORMAT_ARGB8888, SDL_TEXTUREACCESS_STATIC,
            GLCD_DISPWIDTH,
            GLCD_DISPHEIGHT);
}


/* Vorschriftsmaessige Beendigung vorbereiten */
void mock_glcd_deinit(void)
{
    SDL_DestroyTexture(pTexture);
    SDL_DestroyRenderer(pRenderer);
    SDL_DestroyWindow(pWindow);
    SDL_Quit();
}


/* Reset wird einfach mit einem Loeschen des Pixelpuffers "emuliert" - ohne
 * Neuzeichnen. Der Sinn liegt eher in der Vollstaendigkeit als darin,
 * das Verhalten eines Resets zu testen. */
void mock_glcd_reset(void)
{
    uint32_t nBytes = GLCD_DISPWIDTH*GLCD_DISPHEIGHT * sizeof(uint32_t);
    memset(pixelbuffer, 255, nBytes);
}



/* Die Display-Ausgabe wird durch die Darstellung des Framebuffer fuer das echte
 * GLCD in das SDL-Fenster emuliert */
void mock_glcd_draw(void)
{
	// Display-Update
	SDL_UpdateTexture(pTexture, NULL, pixelbuffer, 128 * sizeof(Uint32));

	int i,j, idx;

    // Aus GLCD-Puffer in Pixelpuffer von SDL kopieren
	for(i = 0;i<1024;i++) 
    {
		uint8_t c = gl_framebuffer[i];

		int col, row;
		div_t d = div(i,128);
		row = d.quot;
		col = d.rem;

		for (j=0; j<8; j++) 
        {
			idx = 128*8*row+col + 128*j;

			if (c & 0x01)
				pixelbuffer[idx]=0;
			else
				pixelbuffer[idx]=0x00FFFFFF;
			c>>=1;
		}
	}

	SDL_RenderClear(pRenderer);

	{
		// Bildschirm skalieren
		SDL_Rect DstR;
		DstR.x = 0;
		DstR.y = 0;
		DstR.w = SCREEN_SCALE*128U;
		DstR.h = SCREEN_SCALE*64U;
		SDL_RenderCopy(pRenderer, pTexture, NULL, &DstR);
	}

	SDL_RenderPresent(pRenderer);
}
