/*
 * mock_input.c
 *
 * Einfache Emulation des Verhaltens des Eingabegeraets ueber SDL2
 *
 * Der Drehgeber wird ueber die Tastatur emuliert. Einer Rechtsdrehung
 * entspricht die Pfeil-Hoch-Taste, einer Linksdrehung die Pfeil-Herunter-Taste.
 * Dem Tastendruck entspricht die Enter-Taste.
 *
 * Der Event-Poll beendet das Programm mit exit(), falls das Fenster geschlossen
 * wird.
 *
 * Created: 20.04.2015
 *  Author: DL1DOW
 */



#include <stdint.h>
#include <stdio.h>
#include <stdbool.h>
#include <SDL2/SDL.h>
#include "../../main.h"
#include "mock_input.h"
#include "../src_avrcom/platform.h"



/* Der Stub ist nur fuer den PC und wurde nicht mit allen Compilern getestet */
#ifndef PLATFORM_PC
     #error "This stub is only for PC platform"
#endif



/* Globale Variablen */
uint_fast8_t sdlenc = 0; // Encoder-Emulation
bool sdlkey = false; // Taster-Emulation

const uint8_t *keystate;

/* Wenn ein SDL-Fenster offen ist, bedarf die Tastatureingabe keiner
 * Initialisierung.  */
void mock_input_init(void)
{
	keystate = SDL_GetKeyboardState(NULL);
}


/* Es ist keine Deinitialisierung notwendig. */
void mock_input_deinit(void)
{
	;
}

/* Es ist kein Reset vorgesehen */
void mock_input_reset(void)
{
	;
}


extern volatile uint_fast16_t sgl_time; // Dauer Tastendruck

/* Eingabegeraete des muEDM durch Tastatureingaben emulieren */
void mock_input_poll(void)
{

    SDL_Event event;
    
    
    //SDL_WaitEvent(&event);
    while(SDL_PollEvent(&event)) 
    {

        switch(event.type)
        {
            case SDL_QUIT:
            	printf("Window closed\n");
            	exit(0);
                break;
            case SDL_KEYDOWN:
                ;
				
                // Keystate wird durch SDL_PumpEvents()/SDL_PollEvent upgedatet


                // Pfeiltasten emulieren Drehgeber
                if (keystate[SDL_SCANCODE_UP] || keystate[SDL_SCANCODE_RIGHT]) 
                {
                    sdlenc++;
                }
                if (keystate[SDL_SCANCODE_DOWN] || keystate[SDL_SCANCODE_LEFT]) 
                {
                    sdlenc--;
                }

                // Return simuliert kurzen Tastendruck
                if (keystate[SDL_SCANCODE_RETURN]) 
                {
                    sgl_time+= 1;
                }

                //Shift+Return simuliert langen Tastendruck
                if ( keystate[SDL_SCANCODE_RETURN] && \
                		(keystate[SDL_SCANCODE_LSHIFT] || keystate[SDL_SCANCODE_RSHIFT]) )
                {
                    sgl_time+= T_LONGKEYPRESS+1;

                }



                break;

        }
    }
}
