/*
 * FTMctrl-Mock-up
 * SDL-Funktionen fuer Emulation
 *
 * Created: 22.12.2014
 *  Author: DL1DOW
 */

#include <stdint.h>
#include <stdio.h>
#include <stdbool.h>
#include <SDL2/SDL.h>
#include "../src_avrcom/platform.h"
#include "../src_mockup_sdl/mock_input.h"
#include "../src_mockup_sdl/mock_glcd.h"
#include "../src_mockup_sdl/mock_timer.h"
#include "mock_sdl.h"



/* Der Stub ist nur fuer den PC und wurde nicht mit allen Compilern getestet */
#ifndef PLATFORM_PC
     #error "This stub is only for PC platform"
#endif


/* Groesse des emulieren Displays */
#define GLCD_DISPWIDTH  128
#define GLCD_DISPHEIGHT 64

/* Skalierung der Bildschirmdarstellung:
 * 1U : Screenshots fuer Doku
 * 2U : Gut fuer Bildschirmdarstellung */
#define SCREEN_SCALE 2U

void sdl_init(void)
{
    
    SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);

	mock_glcd_init();
	mock_input_init();
	mock_timer_init();
}


void sdl_deinit(void)
{
	mock_input_deinit();
    mock_glcd_deinit();
}



/* Loop-Funktion fuer Input-Polls in allen Hauptschleifen */
int loop(void) 
{
	
	mock_event_poll();
	return 1;
}


/* Diese Funktion wird innerhalb des Kontrollflusses von allen Funktionen
 * gepollt, die Zugriff auf Hardware haben, z.B. Dregeber und GLCD. Nicht immer
 * wird ein Update vollzogen */
void mock_event_poll(void)
{
	static uint32_t nextRedraw = 0;
	uint32_t tick = SDL_GetTicks();

	if(tick > nextRedraw)
	{
        mock_input_poll();
        //mock_glcd_draw();    
		nextRedraw = tick + 50;
	}
}


/* Ein paar Variablen als GPIO-Emulation */
uint8_t PORTA;
uint8_t PORTB;
uint8_t PORTC;
uint8_t PORTD;

