/*
 * uart_avr.c
 *
 * UART fuer AVR. Getestet bislang nur fuer ATmega 32
 * Angelehnt an https://www.mikrocontroller.net/articles/AVR-GCC-Tutorial/Der_UART
 *
 * Created: 22.12.2015 12:23:37
 *  Author: Nicolas
 */ 



#include "uart.h"
#define BAUD UART_BAUDRATE

#include <avr/io.h>
#include <util/setbaud.h>

#ifndef F_CPU
    /* F_CPU muss ueber das Projekt definiert sein */
    #warning "Symbol F_CPU undefined"

#endif                         



            


/* Initialisierung ATmega16/ATmega32 */
void uart_init(void)
{

    /* UBRRH_VALUE und UBRR_VALUE werden von setbaud.h berechnet. */
    UBRRH = UBRRH_VALUE;
    UBRRL = UBRRL_VALUE;

    #if USE_2X
        /* U2X-Modus erforderlich */
        UCSRA |= (1 << U2X);
    #else
        /* U2X-Modus nicht erforderlich */
        UCSRA &= ~(1 << U2X);
    #endif
 

    UCSRB |= (1<<TXEN);  // UART TX einschalten
    UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);  // Asynchron 8N1 
}


/* Initialisierung ATmega16/ATmega32 */
int uart_putchar(char c)
{
    while (!(UCSRA & (1<<UDRE))) {} /* warten bis Senden moeglich */
    
    UDR = c;                      /* sende Zeichen */
    return 0;
}



/* Strings auf LCD ausgeben */
void uart_putstr(char *zeichenkette)
{
    while (*zeichenkette) 
    {
        uart_putchar(*zeichenkette);
        zeichenkette++;
    }
}



/* Strings aus Flash auf LCD ausgeben */
void uart_putstr_P(constflash char *zeichenkette)
{
    char temp = *zeichenkette; // Hauptsache nicht Null
    uint8_t i = 1;

    while (temp) 
    {
        uart_putchar(temp);
        temp = *(zeichenkette+i);
        i++;
    }
}