/*
 * infoscreen.c
 *
 * Kleine Hilfsfunktionen wie Statusanzeige oder Logodarstellung
 *
 * Created: 15.07.2013
 *  Author: Nicolas
 */ 


#include "ui_infoscreen.h"
#include <string.h>
#include "../src_avrcom/pgmspace.h"
#include "../src_avrcom/delay.h"
#include "../src_glcd/glcd_generic.h"
#include "../src_glcd/glcd_textfun.h"
#include "../src_ui/bitmaps.h" // Logodatei
#include "../src_encoder/key.h"
#include "../../main.h"
#include "../../svn_version.h"




/* Darstellung des total brillianten EDM-Logos */
void splashscreen(void)
{
	glcd_setpos(0);
	for (uint16_t i=0; i<1024; i++) 
    {
		glcd_putdat(pgm_read_byte(&bitmap_logo[i+2]));
	}
  
    glcd_drawnow();
}


/* Splashscreen darstellen und auf Tastendruck warten */
void splashscreen_test(void)
{
    splashscreen();
    while(!key_getstroke()) delay_ms(10);
}


/* Informationen zum Firmwarestand */
void infoscreen(void)
{
	glcd_clear();
	
	glcd_setpos(129);
	for (uint8_t i = 0; i<127;i++) glcd_putdat(0x81);
  
	#if (defined(__AVR_ATmega32__))
	    #define MCUSTR "ATmega32"
        #define X_GCC "AVR-GCC:"
	#elif (defined(__AVR_ATmega16__))
	    #define MCUSTR "ATmega16"
        #define X_GCC "AVR-GCC:"
	#elif (defined(__AVR_ATmega644__))
	    #define MCUSTR "ATmega644"
        #define X_GCC "AVR-GCC:"
    #elif (defined(STM32F103V8))
        #define MCUSTR "STM32F103 V8"
        #define X_GCC "ARM-GCC:"
    #elif (defined(__clang__))
        #define MCUSTR "x86/i64"
        #define X_GCC "Clang:  "
    #elif defined _MSC_VER || defined __MINGW64__ || \
        defined __linux__ && __linux==1
        #define MCUSTR "x86/i64"
	    // Leerzeichen wichtig fuer Ausrichtung
        #define X_GCC "GCC:    "
    #else
  	    #define MCUSTR "unknown"
        #define X_GCC "unknown:"
	#endif
  
    #ifndef PROJECTNAME
        #define PROJECTNAME "EDM by DL1DOW"
    #endif
  
  
    #define INFOSTRING1 PROJECTNAME "\n\n" \
        "build:  " __DATE__ "\n" \
        "        " __TIME__ "\n" \
        "SVN:    " SVN_REVISION  "\n" \
        "MCU:    " MCUSTR "\n"

     
	glcd_setpos(0);
	glcd_putstr_P(PSTR(INFOSTRING1));
	strncpy_P(glstr_Buf,PSTR(X_GCC "\033v " __VERSION__ ),40);
	glstr_Buf[20] = '\0';// Darstellungsende; ARM-GCC-Versionsname sonst zu lang
    glcd_putstr(glstr_Buf);
    glcd_putstr_P(PSTR("\033n\033f"));

    glcd_drawnow();
  
	while(!key_getstroke()) delay_ms(10);
	
}
