/*
 * ioMapping.h
 * Pinbelegung des gesamten Projekts
 */ 


#ifndef IOMAPPING_H
    #define IOMAPPING_H

    /* ******************************************************************************** */
    /*  Grafik-LCD                                                                      */
    /* ******************************************************************************** */
    /* Grafik-LCD Steuer-Port: */
    #define GLCD_KS0108_DI_GPIO    (&PORTC)
    #define GLCD_KS0108_DI_Pin     (1U<<PC2)  // Data/Instr (D=1/I=0)
    #define GLCD_KS0108_RW_GPIO    (&PORTC)
    #define GLCD_KS0108_RW_Pin     (1U<<PC3)  // Read/Write (R=1/W=0)
    #define GLCD_KS0108_ECLK_GPIO  (&PORTC)
    #define GLCD_KS0108_ECLK_Pin   (1U<<PC4)  // Enable (E)
    #define GLCD_KS0108_CS1_GPIO   (&PORTC)
    #define GLCD_KS0108_CS1_Pin    (1U<<PC6)  // CS/1
    #define GLCD_KS0108_CS2_GPIO   (&PORTC)
    #define GLCD_KS0108_CS2_Pin    (1U<<PC5)  // CS/2
    #define GLCD_KS0108_RESET_GPIO (&PORTC)
    #define GLCD_KS0108_RESET_Pin  (1U<<PC7)  // Reset-Pin (RESET = 0)

    /* Grafik-LCD Daten-Port   */
    /* Px7: -> DB7 (MSB) + Busy-Flag)
     * Px6: -> DB6
     * Px5: -> DB5
     * Px4: -> DB4
     * Px3: -> DB3
     * Px2: -> DB2
     * Px1: -> DB1
     * Px0: -> DB0 (LSB)   */
    #define GLCD_KS0108_DATA_GPIO (&PORTB)
    #define GLCD_PWM 0 // GLCD hat keine PWM


    #define GLCD_LIGHT_GPIO (&PORTA)
    #define GLCD_LIGHT_Pin  (1U<<PA4)


    /* ******************************************************************************** */
    /*  Potis und Bedienelemente                                                        */
    /* ******************************************************************************** */
    // Drehgeber und Taster
    #define ENC_A_GPIO      (&PORTA)
    #define ENC_A_Pin       (1<<PA5)
                            
    #define ENC_B_GPIO      (&PORTA)
    #define ENC_B_Pin       (1<<PA6)
                            
    #define TAST_GPIO       (&PORTA)
    #define TAST_Pin        (1<<PA7)


    /* ******************************************************************************** */
    /*  Motorsteuerung                                                                  */
    /* ******************************************************************************** */
    // Motor 1
    #define MOT1A_GPIO      (&PORTC)
    #define MOT1A_Pin       (1U<<PC0)
    #define MOT1B_GPIO      (&PORTC)
    #define MOT1B_Pin       (1U<<PC1)
    #define MOT1_EN_GPIO    (&PORTA)
    #define MOT1_EN_Pin     (1U<<PA0) 
                           
    // Motor 2             
    #define MOT2A_GPIO       (&PORTD)
    #define MOT2A_Pin       (1U<<PD5)
    #define MOT2B_GPIO      (&PORTD)
    #define MOT2B_Pin       (1U<<PD6)
    #define MOT2_EN_GPIO    (&PORTA)
    #define MOT2_EN_Pin     (1U<<PA2)

    // Strommessung
    #define SENSE1_Pin      (1U<<PA1)
    #define SENSE2_Pin      (1U<<PA3)
    
    // Pulssignal
    #define PULSE_IN1_GPIO (&PORTD)
    #define PULSE_IN1_Pin  (1U<<PD4)
    #define PULSE_IN2_GPIO (&PORTD)
    #define PULSE_IN2_Pin  (1U<<PD3)

    // Pulssignal Enable
    #define EN_INPUT_Pin   (1U<<PD2)
    #define EN_INPUT_GPIO  (&PORTD)


    // Freier Pin/General Purpose Pin (GPP)
    #define SPARE0_Pin     (1<<PD7)
    #define SPARE0_GPIO    (&PORTD)

    // ADC-Kanaele
    #define MOT1_ADC_MUX 1
    #define MOT2_ADC_MUX 3


#endif 


