/*
 * menu_tests
 *
 * Men fr Hardware- und Funktionstests
 *
 * Created: 9.02.2014
 *  Author: Nicolas
 */


#include "../../common/src_avrcom/pgmspace.h"
#include "../../common/src_glcd/glcd_generic.h"
#include "../../common/src_glcd/glcd_textfun.h"
#include "../../common/src_glcd/glcd_test.h"
#include "../../common/src_encoder/encoder_test.h"
#include "../../common/src_encoder/key_test.h"
#include "../../common/src_avrcom/delay_test.h"
#include "../../common/src_adc/adc_test.h"
#include "../../common/src_misc/memdump.h"   // sramdump(), flashdump(), eepromdump()
#include "../../common/src_ui/ui_infoscreen.h"
#include "../../common/src_misc/crc_test.h"
#include "../../common/src_ui/bitmaps.h"
#include "../../common/src_misc/clock_test.h"
#include "../../project/src_rotctrl/ftm_eeprom.h"

#include "../../common/src_ui/locale.h"
#include "../../project/src_ui/locale_prj.h"
#include "../../common/src_menu/menu.h"


void menu_msgbox_test(void);



/* Menue Hilfsfunktionen
 *
 * Hilfsfunktionen sind Funktionen, die spaeter (an anderer Stelle) von der Firmware 
 * unveraendert gebraucht werden. */
static constflash menudata_t Menu_hlpfcn_dat[] =
{
    {MT_RETURN        ,voidFcn,{.pvoid=NULL}                        ,0,0,0},
    {MT_INFOSCREEN    ,voidFcn,{.pvoidFcn=infoscreen}               ,0,0,0},
    {MT_SRAMDUMP      ,voidFcn,{.pvoidFcn=sramdump}                 ,0,0,0},
    {MT_FLASHDUMP     ,voidFcn,{.pvoidFcn=flashdump}                ,0,0,0},
    {MT_EEPROMDUMP    ,voidFcn,{.pvoidFcn=eepromdump}               ,0,0,0},
    {MT_SPLASHSCREEN  ,voidFcn,{.pvoidFcn=splashscreen_test}        ,0,0,0},
    // Nullzeile:
    {MT_NULL          ,nothing,{.pvoid=NULL}                        ,0,0,0},
};


/* Menue Testfunktionen
 *
 * Testfunktionen dienen allein dem Test einzelner Softwarekomponenten und werden im 
 * eigentlichen Programmablauf spaeter nicht mehr benoetigt. */
static constflash menudata_t Menu_tstfcn_dat[] =
{
    {MT_RETURN        ,voidFcn,{.pvoid=NULL}                        ,0,0,0},
    {MT_TEST_EERESTORE,voidFcn,{.pvoidFcn=test_ftm_eep_read}        ,0,0,0},
    {MT_MESSAGE_TEST  ,voidFcn,{.pvoidFcn=menu_msgbox_test}         ,0,0,0},
    {MT_TEST_TIMEOUT  ,voidFcn,{.pvoidFcn=timeout_test_visual}      ,0,0,0},
    {MT_TEST_CRC16    ,voidFcn,{.pvoidFcn=crc16_test}               ,0,0,0},
    {MT_TEST_ADC      ,voidFcn,{.pvoidFcn=adc_test}                 ,0,0,0},
    {MT_TEST_ENCODER  ,voidFcn,{.pvoidFcn=encoder_test}             ,0,0,0},
    {MT_TEST_KEY      ,voidFcn,{.pvoidFcn=key_test}                 ,0,0,0},
    {MT_TEST_TEXT     ,voidFcn,{.pvoidFcn=glcd_texttest}            ,0,0,0},
    {MT_TEST_DELAY    ,voidFcn,{.pvoidFcn=delay_test}               ,0,0,0},
    // Nullzeile:                                                     
    {MT_NULL          ,nothing,{.pvoid=NULL}                        ,0,0,0},
};



/* Darstellung Kopfzeile */
static void head_menu_hlpfcn(void)
{
    glcd_setpos(0);

    // Schwarzer Hintergrund  
    uint8_t i = 0;
    do 
    {
        glcd_putdat(0xFF);
    } while(++i);

    // Ueberschrift
    glcd_setpos(144);
    glcd_putstr_P(LT_HEADLINE_HELPFCN);
}


/* Darstellung Kopfzeile */
static void head_menu_tstfcn(void)
{
    glcd_setpos(0);

    uint8_t i = 0;
    do 
    {
        glcd_putdat(0xFF);
    } while(++i);

    glcd_setpos(144);
    glcd_putstr_P(LT_HEADLINE_TESTFCN);
}


static constflash menuconfig_t Menu_hlpfcn_config = 
{
    2, // headheight
    NULL, // headtext
    head_menu_hlpfcn //pheadfkt
};

static constflash menuconfig_t Menu_tstfcn_config = 
{
    2, // headheight
    NULL, // headtext
    head_menu_tstfcn //pheadfkt
};




/* Menue mit Hilfsfunktionen */
void menu_hlpfcn(void)
{
    menu_standard(Menu_hlpfcn_dat,&Menu_hlpfcn_config);
}

/* Menue mit vielen Tests */
void menu_tstfcn(void)
{
    menu_standard(Menu_tstfcn_dat,&Menu_tstfcn_config);
}



/* Test von Auswahlfenstern ("Popup") */
void menu_msgbox_test(void)
{
    uint16_t status;
    usermessageitem_t items[]  = {MT_YES,MT_NO,MT_CANCEL,MT_NULL};   
    usermessage_t message;

    message.text = LT_ERRORMSG;
    message.items = items;
    message.headheight = 3;
    message.icon = icon_error;
    message.errcode = 10;
  
    status = usermessage(&message);
  
    printf_P(PSTR("n = %u\a"),status);
}   


