/*
 * ftm_calc.c
 *
 * Berechnungen fuer Rotorsteuerung
 *
 * Created: 15.02.2016
 *  Author: Nicolas
 */ 


#include "ftm_calc.h"
#include "../../common/src_base/datatypes.h"
#include "../../project/src_rotctrl/rot_variables.h"
#include "../../project/src_rotctrl/rot_settings.h"


/* Winkel von Grad in Pulse umrechnen */
int16_t deg2pulse(int16_t deg, uint8_t ppr)
{
    
    //if (deg > 0) {
        //return (deg*ppr + 180L)/360L;
    //}
    //else {
        //return (deg*ppr - 180L)/360L;
    //}
    
    int_fast24_t temp = deg;
    temp *=ppr;
    temp += temp < 0 ? -180 : 180;
    temp /= 360;
    
    return temp;
}



/* Winkel von Pulse in Grad umrechnen */
int16_t pulse2deg(int16_t pulse, uint8_t ppr)
{
    //if (pulse>0) {
        //return (pulse*360 + ppr/2)/ppr;
    //}
    //else {
        //return (pulse*360 - ppr/2)/ppr;
    //}    
    
    int_fast24_t temp = pulse;
    temp *= 360;
    temp += temp < 0 ? -ppr/2 : ppr/2;
    temp /= ppr;
    
    return temp;
}



/* Einmalige Daten berechnen (groesstenteils aus EEPROM abgeleitete Werte) */
void calc_init(void)
{
    rot_settings_t Rs = GlSetting;
    for (uint8_t i=0; i<2; i++)
    {
        // Auswirkung der Winkelgrenzen (EEPROM-Einstellung)
        Glcstat.mot[i].degMax = 360L + Rs.mot[i].limitDeg + Rs.mot[i].stepoverDeg;
        Glcstat.mot[i].degMin = Rs.mot[i].limitDeg - Rs.mot[i].stepoverDeg;
    
        // Einstellungn auch in "userstate" kopieren:
        Glustat.mot[i].degMax = Glcstat.mot[i].degMax;
        Glustat.mot[i].degMin = Glcstat.mot[i].degMin;
        
        // Bei Start ist Soll = Ist
        uint8_t ppr = GlSetting.mot[i].pulsePerRev;
        Glcstat.mot[i].deg = pulse2deg(Glcstat.mot[i].pulse, ppr);        
        Glustat.mot[i].deg = Glcstat.mot[i].deg;
    }    

    // Einstellungen der Steuerungsart
    switch (Rs.screen)
    {
        case 0: // Fallthru
        case 2:
            Glustat.usrctrl = ctrl_motion;
            break;
            
        case 1: // Fallthru
        case 3:
            Glustat.usrctrl = ctrl_angle;
    }
}



/* Laufende Daten in Glcstat berechnen */
void calc_update(ctrlstat_t * const Cstat)
{
    for (uint8_t i=0; i<2; i++) 
    {
        // Motorstrom
        Cstat->mot[i].current = (Cstat->mot[i].adc * \
            DIGIT_CURRENT_ENUM)/DIGIT_CURRENT_DENOM;

        #ifdef PLATFORM_PC
            // Am Mock-Up am PC werden die Winkeldaten nicht aktualisiert, um eine
            // Bewegung per Tasten zu ermoeglichen
            
        #else
            // Rotorposition
            uint8_t ppr = GlSetting.mot[i].pulsePerRev;
            Cstat->mot[i].deg = pulse2deg(Cstat->mot[i].pulse, ppr);
        #endif
    }
}

