/*
 * ftmctrl.h
 *
 * FTM25/6-Rotorsteuerung: Statusmaschine und Haupt-Steuerroutinen
 *
 * Created: 16.09.2015
 *  Author: Nicolas
 */ 



#include "ftm_ctrl.h"
#include <stdio.h>

#include "../../common/src_base/assert.h"
#include "../../common/src_base/intmath.h"
#include "../../common/src_avrcom/delay.h"

#include "../../common/src_glcd/glcd_textfun.h"
#include "../../common/src_misc/clock.h"
#include "../../common/src_encoder/key.h"
#include "../../common/src_encoder/encoder_panasonic.h"

#include "../../project/src_adc/adc_multiplex.h"
#include "../../project/src_rotctrl/dcmotor.h"
#include "../../project/src_rotctrl/pulsecounter.h"
#include "../../project/src_rotctrl/rot_variables.h"
#include "../../project/src_rotctrl/rot_settings.h"
#include "../../project/src_rotctrl/ftm_calc.h"
#include "../../project/src_rotctrl/ftm_eeprom.h"
#include "../../project/src_rotctrl/ftm_screens.h"
#include "../../project/src_ui/locale_prj.h"


#include "../../main.h"      // Einstellungen
#include "../../ioMapping.h"






#ifndef T_LONGKEYPRESS
    #error "missing define from main.h"
#endif





/* ************************************************************************************ *
 *  Benutzereingaben                                                                    *
 * ************************************************************************************ */

// Tasten-Events
typedef enum
{
    keypress_none,
    keypress_short,
    keypress_long
} keyevent_t;



/* Bedienelemente zuruecksetzen */
void userinput_init(void)
{
	encoder_get_diff();
    
	key_gettime();
    key_gettime();
}



/* Abfrage der Taster-Hardware (Drehgeber und Taster) */
void userinput_get(keyevent_t * pkeyevent, int8_t * delta)
{

    /* Abfrage der Taster-Hardware: Drehgeber */
    *delta = encoder_get_diff();
        
    /* Abfrage der Taster-Hardware: Taster (Kurzen oder langen Tastendruck 
        * feststellen) */
    uint16_t tKey = key_gettime();

    if (tKey>T_LONGKEYPRESS)
    {
        *pkeyevent = keypress_long;
        printf("long");
    }
    else if (tKey > 0)
    {
        *pkeyevent = keypress_short;
        printf("short");
    }
    else
    {
        *pkeyevent = keypress_none;
    }
}



/* Tastendruecke interpretieren und Status der Benutzerschnittstelle (= SOLL-Zustand) 
 * anpassen */
void statemachine_userinput_update(const ctrlstat_t * Cstat, ctrlstat_t * Ustat)
{
    int8_t deltaVal;
    keyevent_t keyevent;
    
    userinput_get(&keyevent,&deltaVal);
    
    int_fast8_t enc_state;
    uint8_t active = Ustat->activeUsr;  
     
    switch (Ustat->usrctrl)
    {
        case ctrl_motion:
            // Bewegungssteuerung links/rechts/stop

            // Bei Bewegungssteuerung ist immer der Motor aktiv, der auch gesteuert wird.
            Ustat->activeMot = active;

            // Drehgeber auswerten (nur Aenderungen)
            // Erlaubt: -2, 0 2
            // Hier wird auch der Int-Repraesentierung von motion_t herummanipuliert
            
            enc_state = Ustat->mot[active].motion/2;
            enc_state = limadd_int8(enc_state,deltaVal,-1,1);
            Ustat->mot[active].motion = enc_state*2;
            
            // Im Fall des Ueberlaufs anhalten und Bewegung in diese Richtung nicht
            // mehr erlauben
            switch(Ustat->mot[active].motion) 
            {
                case motor_left:        // Fallthru
                case motor_runout_left:
                    // Keine Linksdrehung mehr erlauben, wenn am "linken Anschlag"
                    if (   Ustat->mot[active].deg < Ustat->mot[active].degMin ) 
                    {
                        Ustat->mot[active].motion = motor_stop;
                    }
                    break;
                
                case motor_stop:
                    break;
                
                case motor_runout_right: // Fallthru
                case motor_right:
                    // Keine Rechtsdrehung mehr erlauben, wenn am "rechten Anschlag"
                    if (   Ustat->mot[active].deg > Ustat->mot[active].degMax ) 
                    {
                        Ustat->mot[active].motion = motor_stop;
                    }
                    break;
                
            }         
            
            break;
            
        case ctrl_angle:
            if (Cstat->mot[active].err)
            {
                // Im "Fehlerfall" ist die Winkelsteuerung inaktiv
                Ustat->mot[active].deg = Cstat->mot[active].deg;
            }
            else
            {
                // Winkelvorgabe: Gradzahl direkt verstellen
                Ustat->mot[active].deg = limadd_int16(Ustat->mot[active].deg,5*deltaVal,\
                    Ustat->mot[active].degMin,Ustat->mot[active].degMax);
            }
            break;
                            
        case ctrl_finish:
            // Steuerung wartet nur noch auf ihr baldiges Ende. Da kommt niemand
            // mehr zwischen
            ;
    }
     

    /* Tastendruecke interpretieren */
    /* Tastendruecke gewinnen immer gegen den Drehgeber */    
    switch (keyevent) 
    {
        case keypress_none:
            /* Kein Tastendruck - Nichts passiert */
            break;
        
        case keypress_short:
            /* Mit kurzem Tastendruck aktiven Motor umschalten */

			/* Nach dem Umschalten muss der Motor wieder stehen */
			Ustat->mot[Ustat->activeUsr].motion = motor_stop;

            // Umschalten zwischen 0 und 1
            Ustat->activeUsr = 1-Ustat->activeUsr;

            break;
        
        case keypress_long:    
            /* Mit langem Tastendruck Motorsteuerung verlassen */
            Ustat->activeMot = 0;
            Ustat->activeUsr = 0;
            
            // Im links-/rechts-/stop-Modus Motoren anhalten
            Ustat->mot[0].motion = motor_stop;
            Ustat->mot[1].motion = motor_stop;
            
            // Fuer Winkelsteuerung Ist -> Soll (fuehrt auch zum anhalten)
            Ustat->mot[0].deg = Cstat->mot[0].deg;
            Ustat->mot[1].deg = Cstat->mot[1].deg;
            Ustat->usrctrl = ctrl_finish;
            break;
            
        default:
            assert(false);

    }    
}



/* Rotor testen: 
 * Wenn der Motor laufen soll, sollte der Rotor auch Strom aufnehmen und Pulse 
 * zurueckgeben. Ist eines von beidem nicht der Fall, ist nur noch der manuelle Modus 
 * moeglich */
void statemachine_check_motorstat(ctrlstat_t * Cstat, ctrlstat_t * Ustat)
{
    (void) Ustat;
    uint16_t noRotorTimeout = NO_ROTOR_TIMEOUT_MS; // [ms]
    
    static timeoutstor_t timer[2];  // Statusvariable Timeout
    static int16_t  last_pulse[2];
    bool gotpulse, gotcurrent;
    
    for (uint8_t i=0;i<2; i++)
    {
        switch (Cstat->mot[i].motion)
        {
            case motor_stop:         // Fallthru
            case motor_runout_left:  // Fallthru
            case motor_runout_right:
                // Beim allerersten Start sind immer beide Motoren gestoppt
                setTimeout(&timer[i],noRotorTimeout);
                break;
            
            case motor_left:  // Fallthru
            case motor_right: 
                
                // Pruefen, ob ein Impuls und Strom gemessen wurden
                gotpulse = last_pulse[i] != Cstat->mot[i].pulse;
                gotcurrent = Cstat->mot[i].current > 0;
                if (gotpulse && gotcurrent)
                {
                    // Fehler verhindern
                    setTimeout(&timer[i],noRotorTimeout);
                    
                    // Wenn nach Fehler wieder Strom und Puls gemessen werden, wieder
                    // normalen Betrieb aufnehmen
                    Cstat->mot[i].err = 0;
                }
                else
                {
                    if(pollTimeout(&timer[i]))
                    {
                        // Wenn kein Motor erkennbar, anhalten nach Timeout. Im 
                        // Handbetrieb kann der Benutzer das jederzeit wieder 
                        // ueberstimmen.
                        Cstat->mot[i].err = 1;
                        Cstat->mot[i].motion = motor_stop;
                        Ustat->mot[i].motion = motor_stop;
                    }
                }
                
                break;
        }
        
        // Sichern fuer Erkennung der Aenderung
        last_pulse[i]  = Cstat->mot[i].pulse;
    }
    
}



/* Aktuelle Rotorposition ins EEPROM sichern: 3 Sekunden, nachdem beide Rotoren gestoppt 
 * sind, ein einziges Mal */
void statemachine_save_position_data(ctrlstat_t * Cstat)
{
    static timeoutstor_t timeout;  // Statusvariable Timeout
    
    if ( !((Cstat->mot[0].motion == motor_stop) && (Cstat->mot[1].motion == motor_stop)) )
    {
        // Solange auch nur ein Motor dreht: Timeout aktiv halten, Status "busy"
        setTimeout(&timeout,SAVE_EEPROM_AFTER_MS);
        Cstat->busy = true;
    }
    else if ( Cstat->busy && (pollTimeout(&timeout) || Cstat->usrctrl == ctrl_finish ) )
    {
        // Beide Motoren stehen, EEPROM nicht nicht geschrieben, Timeout abgelaufen
        savePulse();
        Cstat->busy = false;
    }
}


/* Zustandsmaschine ist dafuer verantwortlich, den Benutzerwunsch (Ustat) auf erlaubtem 
 * Weg auf den Systemzustand (Cstat) zu uebertragen */
void statemachine_motorctrl_update(ctrlstat_t * Cstat, ctrlstat_t * Ustat)
{
    static timeoutstor_t timeout;  // Statusvariable Timeout

    
    /* Zustandsmaschine zwischen Soll- und Istzustand wechseln */
    uint8_t act;

    if
        (
            (Cstat->mot[0].motion != Ustat->mot[0].motion) ||
            (Cstat->mot[1].motion != Ustat->mot[1].motion) ||
            (Cstat->activeMot     != Ustat->activeMot)     ||
            (Cstat->usrctrl       != Ustat->usrctrl)
        )
    {
        // Zustandswechsel
        act = Cstat->activeMot;
        switch (Cstat->mot[act].motion)
        {
            case motor_stop:
                // Stillstand. Von hier kann jeder Zustand sicher erreicht
                // werden
                Cstat->mot[0].motion  = Ustat->mot[0].motion;
                Cstat->mot[1].motion  = Ustat->mot[1].motion;
                Cstat->activeMot      = Ustat->activeMot;
                Cstat->usrctrl        = Ustat->usrctrl;  // zum Beenden
                break;
            
            case motor_runout_left:  // Fallthru
            case motor_runout_right:
                // Der Auslauf kann nur per Timeout verlassen werden
                if(pollTimeout(&timeout))
                {
                    // Motor abschalten, Auslauf beenden
                    Cstat->mot[act].motion = motor_stop;
                }
                break;
            
            case motor_left:         // Fallthru
                // Normaler Lauf -> In Auslauf wechseln
                Cstat->mot[act].motion = motor_runout_left;
                setTimeout(&timeout,GlSetting.runoutTimeMs);
                break;
            
            case motor_right:
                // Normaler Lauf -> In Auslauf wechseln
                Cstat->mot[act].motion = motor_runout_right;
                setTimeout(&timeout,GlSetting.runoutTimeMs);
                break;
        }
    }
}



/* Zustandsmaschine fuer Regelung nach Winkelvorgabe (Soll/Ist)
 * Diese Regelung passt lediglich den "Benutzerwunsch" an, d.h. uebersetzt den Wunsch 
 * nach einem Zielwinkel in einen "links/rechts/stop"-Benutzerwunsch. Ein unerlaubter
 * Winkelbereich ist schon in Ustat ausgeschlossen. Aus Cstat wird nur gelesen */
void statmachine_anglectrl_update(const ctrlstat_t * Cstat, ctrlstat_t * Ustat)
{
    // Diese Zustandsmaschine ist nur fuer Winkelsteuerung verantwortlich
    if (!(Ustat->usrctrl == ctrl_angle))
    {
        return;        
    }

    // Nur ein Motor wird gesteuert
    uint8_t i = Cstat->activeMot;
    
    // Pruefen, ob IST-Winkel innerhalb der Toleranz:
    int8_t tol = 3;
    if (Cstat->mot[i].deg < Ustat->mot[i].deg - tol)
    {
        // Ist<Soll
        switch (Ustat->mot[i].motion)
        {
            case motor_left:
                // anhalten
                Ustat->mot[i].motion   = motor_stop;
                break;
                
            case motor_stop:
                // In Soll-Richtung drehen: (anderen Motor verriegeln)
                Ustat->mot[i].motion   = motor_right;
                Ustat->mot[1-i].motion = motor_stop;
                break;

            case motor_right:
                // weiterdrehen lassen (anderen Motor verriegeln)
                Ustat->mot[1-i].motion = motor_stop;
                break;
                
            default:
                assert(false);
        }
    }
    else if (Cstat->mot[i].deg > Ustat->mot[i].deg + tol)
    {
        // Ist>Soll
        switch (Ustat->mot[i].motion)
        {
            case motor_left:      
                // weiterdrehen lassen  (anderen Motor verriegeln)
                Ustat->mot[1-i].motion = motor_stop;
                break;
            
            case motor_stop:
                // In Soll-Richtung drehen: (anderen Motor verriegeln)
                Ustat->mot[i].motion = motor_left;
                Ustat->mot[1-i].motion = motor_stop;
                break;

            case motor_right:
                // anhalten
                Ustat->mot[i].motion = motor_stop;
                break;
                
            default:
                // Fehler: Zustaende "motor_runout_xx" duerfen in Ustat nicht vorkommen
                assert(false);
        }
    }
    else
    {
        // Wenn innerhalb Toleranz, Motor anhalten und den Aktiv-Token weitergeben
        Ustat->mot[i].motion = motor_stop;
        Ustat->activeMot = 1-i;
    }
}



/* Haupt-Steuerschleife fuer die Rotorsteuerung */
void control_ftm(void)
{

    dcmotor_t *pmot[2];      // Statusvariablen DC-Motoren
    
    // Zaehler zuruecksetzen
    userinput_init();
    calc_init();
    
    
    // DC-Motoren Initialisieren
    {
        dcmotorinit_t mot_init[2] =
        {
            {
                .GPIOA = MOT1A_GPIO, .PinA = MOT1A_Pin,
                .GPIOB = MOT1B_GPIO, .PinB = MOT1B_Pin,
                .GPIOEn = MOT1_EN_GPIO, .PinEn = MOT1_EN_Pin,
            },
            {
                .GPIOA = MOT2A_GPIO, .PinA = MOT2A_Pin,
                .GPIOB = MOT2B_GPIO, .PinB = MOT2B_Pin,
                .GPIOEn = MOT2_EN_GPIO, .PinEn = MOT2_EN_Pin,
            }
        };        

        pmot[0] = dcmotor_init(&mot_init[0]);
        pmot[1] = dcmotor_init(&mot_init[1]);
    }    


    
    while(loop()) 
    {
        // Benutzerwunsch abfragen
        statemachine_userinput_update(&Glcstat,&Glustat);
        
        // Daten aus nicht-synchronen Funktionen (z.B. Pulszaehlung, ADC) 
        // uebernehmen und Folgewerte berechnen
        calc_update(&Glcstat);          
        
        // Winkelsteuerung (wenn noetig)        
        statmachine_anglectrl_update(&Glcstat,&Glustat);
        
        // make
        statemachine_check_motorstat(&Glcstat, &Glustat);
        
        // Motorsteuerung
        statemachine_motorctrl_update(&Glcstat,&Glustat);

        // Motoren ansteuern: Ist-Zustand Realitaet werden lassen, Richtungseinstellung
        dcmotor_control(pmot[0],Glcstat.mot[0].motion * GlSetting.mot[0].dir);        
        dcmotor_control(pmot[1],Glcstat.mot[1].motion * GlSetting.mot[1].dir);        
        
        // Zustand darstellen
        display_update();

        // Positionsdaten sichern
        statemachine_save_position_data(&Glcstat);

        #ifdef PLATFORM_PC
            // Bei Tests unter Windows/Linux wird die Anwendung etwas schlafen gelegt, um 
            // die Prozessorlast zu verringern
            delay_ms(1); 
        #endif
        
        // Bei Bedarf Programm verlassen - aber erst, wenn EEPROM gesichert
        if ( (Glcstat.usrctrl == ctrl_finish) && !Glcstat.busy )
        {
            break;
        }
    }
	
    // Programm verlassen
    dcmotor_control(pmot[0],motor_stop);
    dcmotor_control(pmot[1],motor_stop);    
    dcmotor_deinit(pmot[0]);
    dcmotor_deinit(pmot[1]);
}
