/*
 * ftm_menu.c
 *
 * Grafische Menues fuer FMT25/6-Rotorsteuerung:
 *  - Hauptmenue
 *  - Einstellungen
 *
 * Created: 18.08.2015
 *  Author: Nicolas
 */




#include "ftm_menu.h"
#include "./rot_settings.h"
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include "../../common/src_glcd/glcd_textfun.h"
#include "../../common/src_glcd/glcd_gfxfun.h"
#include "../../common/src_menu/menu.h"
#include "../../common/src_ui/bitmaps.h"         // Icons
#include "../../common/src_ui/ui_menu_eeprom.h"
#include "../../project/src_ui/locale_prj.h"     // Texte



#include "../../common/src_avrcom/pgmspace.h"
#include "../../project/src_menu/menu_tests.h"
#include "./rot_variables.h"
#include "./ftm_ctrl.h"
#include "./ftm_calc.h"







/* ************************************************************************************ */
/*     Einstellungen-Men                                                               */
/* ************************************************************************************ */
rot_settings_t GlSetting; // Speicherplatz fuer die Einstellungen


/* Darstellung der Men-Kopfzeile */
static void edm_prefsmenu_headline(void) 
{
    glcd_setpos(0);
    glcd_putbitmap_P(icon_spanner);
    glcd_setpos(144);
    glcd_putstr_P(LT_HEADLINE_PREFERENCES);
    glcd_setpos(112);
    glcd_putbitmap_P(icon_spanner);
}


/* Men-Inhalt */
constflash menudata_t calibmenu[] = 
{
    {MT_SAVEANDRET      ,voidFcn,{.pvoid = NULL},0,0, 0}, // Rueckgabewert interessant
    {MT_ABORT           ,voidFcn,{.pvoid = NULL},0,0, 0}, // Rueckgabewert interessant
    {MT_SCREENNO        ,uint8  ,{.puint8=&GlSetting.screen},0,3,1},
    {MT_MOT0DIR         ,int8   ,{.pint8 =&GlSetting.mot[0].dir},-1,1,2},
    {MT_MOT0LIMITDEG    ,int16  ,{.pint16=&GlSetting.mot[0].limitDeg},0 ,360 , 5},
    {MT_MOT0STEPOVERDEG ,int16  ,{.pint16=&GlSetting.mot[0].stepoverDeg},0 ,360 ,5 },
    {MT_MOT0PULSEPERREV ,int16  ,{.pint16=&GlSetting.mot[0].pulsePerRev},0 ,255 ,1},
    {MT_MOT1DIR         ,int8   ,{.pint8 =&GlSetting.mot[1].dir},-1,1,2},            
    {MT_MOT1LIMITDEG    ,int16  ,{.pint16=&GlSetting.mot[1].limitDeg},0 ,360 ,5 },      
    {MT_MOT1STEPOVERDEG ,int16  ,{.pint16=&GlSetting.mot[1].stepoverDeg},0 ,360, 5 },      
    {MT_MOT1PULSEPERREV ,int16  ,{.pint16=&GlSetting.mot[1].pulsePerRev},1 ,255 , 1 },
    {MT_IMAX            ,uint16 ,{.puint16=&GlSetting.iMax},100 ,5000, 100 },
    {MT_RUNOUTTIME      ,uint16 ,{.puint16=&GlSetting.runoutTimeMs},200 ,9900, 100 },   
    {MT_NULL            ,nothing,{.pvoid =NULL},0,0,0}, // Endmarkierung
};


/* Men-Einstellungen */
static constflash menuconfig_t Prefsmenuconfig = 
{
    2, // headheight
    NULL, // headtext
    edm_prefsmenu_headline //pheadfkt
};


/* Auslesen der Daten aus dem EEPROM, Aufruf der Men-Funktion mit den obigen
 * Einstellungen, und geaenderte Daten zurueckspeichern, falls gewuenscht. */
void rot_prefsmenu(void) 
{
    ui_read_eeprom();
    rot_settings_t temp = GlSetting; // Sicherheitskopie

    // Menu anzeigen, Rueckgabewert ist Index des Menueeintrags und kann nur
	// 0 oder 1 sein, da alle anderen Menueeintraege nicht zurueckkehren
    uint8_t res = menu_standard(calibmenu,&Prefsmenuconfig);
 
    //Abbrechen
    if(res) 
    {
        // Sicherheitskopie zurckspielen
        GlSetting = temp;
        return;
    }
	else 
    {
		// Daten korrigieren
		// - keine Korrekturwerte

        ui_write_eeprom();
	}
    
    // Hilfsgroessen berechnen
    // - keine Hilfsgroessen
}




/* ************************************************************************** */
/*     Hauptmen                                                              */
/* ************************************************************************** */


/* Darstellung der Men-Kopfzeile */
static void menuhead(void)
{
    glcd_setpos(0);
    glcd_putbitmap_P(icon_menu);
    glcd_setpos(154);
    glcd_putstr_P(LT_HEADLINE_MAIN_MENU);
    glcd_setpos(112);
    glcd_putbitmap_P(icon_menu);
}


// Hilfsvariablen, um im Menue einen verstellbaren Parameter zu haben
static int16_t mot0ActDeg, mot1ActDeg;

/* Men-Inhalt */
static constflash menudata_t menudat[] =
{
    {MT_RETURN       ,voidFcn,{.pvoidFcn=NULL}           ,0,  0,0},
    {MT_MOT1ACTDEG   ,int16  ,{.pint16 = &mot0ActDeg}    ,0,360,5},
    {MT_MOT2ACTDEG   ,int16  ,{.pint16 = &mot1ActDeg}    ,0,360,5},
    {MT_SCREENNO     ,uint8  ,{.puint8=&GlSetting.screen},0,  3,1},
    {MT_PREFSMENU    ,voidFcn,{.pvoidFcn=rot_prefsmenu}  ,0,  0,0},
    {MT_MENU_HLPFCN  ,voidFcn,{.pvoidFcn=menu_hlpfcn}    ,0,  0,0},
    {MT_MENU_TSTFCN  ,voidFcn,{.pvoidFcn=menu_tstfcn}    ,0,  0,0},
    // Nullzeile (Endmarkierung):
    {MT_NULL         ,nothing,{.pvoid = NULL}            ,0,  0,0},
};


/* Men-Einstellungen */
constflash menuconfig_t Menuconfig =
{
    2, // headheight
    NULL, // headtext
    menuhead //pheadfkt
};


/* Aufruf der Men-Funktion mit den obigen Einstellungen */
void menu_main(void)
{
    
    // Der Winkel wird nur als Puls hinterlegt
    uint8_t ppr0 = GlSetting.mot[0].pulsePerRev;
    uint8_t ppr1 = GlSetting.mot[1].pulsePerRev;
    mot0ActDeg = pulse2deg(Glcstat.mot[0].pulse, ppr0);
    mot1ActDeg = pulse2deg(Glcstat.mot[1].pulse, ppr1);
    
    glcd_init();
    menu_standard(menudat,&Menuconfig);
    
    // Rueckrechnen
    Glcstat.mot[0].pulse = deg2pulse(mot0ActDeg, ppr0);
    Glcstat.mot[1].pulse = deg2pulse(mot1ActDeg, ppr1);
}


