/*
 * ftm_screens.c
 *
 * Darstellung aller Anzeigen der Steuerung auf dem 128x64-Pixel-GLCD
 * Unterstuetzt werden eine Text-Darstellung und eine grafische Darstellung.
 *
 * Created: 08.02.2016 20:40:31
 *  Author: Nicolas
 */ 


#include "ftm_screens.h"
#include "../../common/src_base/assert.h"
#include "../../common/src_base/intmath.h"
#include "../../common/src_misc/clock.h"
#include "../../common/src_glcd/glcd_textfun.h"
#include "../../common/src_glcd/glcd_gfxfun.h"

#include "../../project/src_ui/bitmaps.h"
#include "../../project/src_rotctrl/rot_settings.h"

// Fuer Profiling
#include "../../ioMapping.h"
#include "../../common/src_avrcom/io.h"



/* Hilfsvariablen */
static timeoutstor_t timer; // Timer fuer Animationen


/* Hilfsfunktion: Zeiger fester Groesse zeichnen
 * ang: Richtung in Grad, (0 ist nach oben)
 * offsetX, offsetX: Mittelpunkt */
void draw_needle(int16_t ang, uint8_t offsetX, uint8_t offsetY)
{
    uint8_t x0,y0,x1,y1,x2,y2;
    
    // Eckpunkte
    x0 = offsetX+sin13(ang-90);
    y0 = offsetY+sin13(ang);
    
    x1 = offsetX+sin13(ang+155-90);
    y1 = offsetY+sin13(ang+155);

    x2 = offsetX+sin13(ang+205-90);
    y2 = offsetY+sin13(ang+205);

    // Kanten
    glcd_line(x0,y0,x1,y1);
    glcd_line(x0,y0,x2,y2);
    glcd_line(x1,y1,x2,y2);
}



/* Hilfspunktion: Punkt auf Keisbahn zeichnen */
void draw_dot(int16_t ang, uint8_t offsetX, uint8_t offsetY)
{
    uint8_t x0 = offsetX +sin13(ang-90);
    uint8_t y0 = offsetY +sin13(ang);
    glcd_setPixel(x0,y0);
    glcd_setPixel(x0+1,y0);
    glcd_setPixel(x0-1,y0);
    glcd_setPixel(x0,y0+1);
    glcd_setPixel(x0,y0-1);
    glcd_setPixel(x0+1,y0+1);
    glcd_setPixel(x0-1,y0+1);
    glcd_setPixel(x0+1,y0-1);
    glcd_setPixel(x0-1,y0-1);
}



/* Screen 0: Zeigerdarstellung, nur Himmelsrichtung, links/rechts/stop, busy */
void screen0_init(void)
{
    setTimeout(&timer,0);
}    

void screen0_update(void)
{
    // Daten aus globalen Structs, werden nur gelesen
    ctrlstat_t const * Cstat = &Glcstat;
    ctrlstat_t const * Ustat = &Glustat;
    rot_settings_t const Rs = GlSetting;
    
    static uint8_t iFrame = 0;  // Momentan dargestellte Displayhaelfte
    
    uint8_t temp;
    uint8_t yOffset[2] = {0,64};

    

    // Hintergrund (Displayhaelfte) darstellen - leert auch direkt das Display
    glcd_setpos(yOffset[iFrame]);
    if (Cstat->mot[iFrame].err) 
    {
        glcd_putbitmap_P(rotorError);
    }
    else
    {
        glcd_putbitmap_P(bitmap_background);    
    }
        

        
    // Motorstatus: Bewegung oder Pause
    glcd_setpos(0U+20U+4U+yOffset[iFrame]);
    switch (Cstat->mot[iFrame].motion)
    {
        case motor_left:
            glcd_putstr_P(PSTR("\x11  "));
            break;
                
        case motor_runout_left:   
            // "||" (invertiert)
            glcd_putstr_P(PSTR("\033i \x13 \033i "));
            break;
            
        case motor_stop: 
            glcd_putstr_P(PSTR(" \x13 "));
            break;

        case motor_runout_right:
            // "||" (invertiert)
            glcd_putstr_P(PSTR("\033i \x13 \033i "));
            break;

        case motor_right:
            glcd_putstr_P(PSTR("  \x10"));
            break;
                
    }

    

    // Zeiger (Offset von 360 hier egal)
    draw_needle(Cstat->mot[iFrame].deg,35,32+yOffset[iFrame]);
        

        
    // Zielpunkt (Nur bei Winkelsteuerung)
    if (Ustat->usrctrl == ctrl_angle)
    {
        draw_dot(Ustat->mot[iFrame].deg,35,32+yOffset[iFrame]);
    }

        
        
    // Ueberlauf (im links/rechts/stop-Modus von IST-Wert, im Grad-Modus 
    // SOLL-Wert)
    int16_t ang;
    switch (Ustat->usrctrl) 
    {
        case ctrl_angle:
            ang = Ustat->mot[iFrame].deg;
            break;
            
        case ctrl_motion: // Fallthru
        case ctrl_finish: // Fallthru
        default:
            ang = Cstat->mot[0].deg;
    }
        
    if( ang > (Rs.mot[iFrame].limitDeg + 360) )
    {
        glcd_circle(35,32+yOffset[iFrame],13,Rs.mot[iFrame].limitDeg,ang-360);
    }
    else if ( ang < Rs.mot[iFrame].limitDeg )
    {
        glcd_circle(35,32+yOffset[iFrame],13,ang,Rs.mot[iFrame].limitDeg);
    }
        

    
    
    /* Ab hier: Dinge, die nur einmal auf dem GLCD erscheinen ************************* */
    
    
    // Busy-Zeichen (EEPROM nicht gesichert) in Form einer einfachen Animation
    glcd_setpos(7U*128U+112U);
    if(Cstat->busy)
    {
        static int8_t ii = 0;
    
        if (pollTimeout(&timer))
        {
            ii++;
            if (ii==3) ii=0;
            setTimeout(&timer,250);              
        } 
            
        switch (ii)
        {
            case 0: glcd_putbitmap_P(bitmap_ani0); break;
            case 1: glcd_putbitmap_P(bitmap_ani1); break;
            case 2: glcd_putbitmap_P(bitmap_ani2); break;
        }
    }
    else
    {
        glcd_putbitmap_P(bitmap_ani1);
    }


    // Busy-Zeichen (EEPROM nicht gesichert)
    if(Cstat->busy)
    {
        glcd_setpos(0U*128U+111U);
        glcd_putdat(0xFF); // Ein bischen Abstand
        printf_P(PSTR("\033s\033ibusy\033i\033n"));
    }



    // Aktiv-Balken
    temp = Ustat->activeUsr*64U;

    // oben links
    glcd_setpos(2*128+12U+temp);
    glcd_adddat(0x4F);
    glcd_adddat(0x01);
    glcd_adddat(0x01);
    glcd_adddat(0x01);
    glcd_adddat(0x00);
    glcd_adddat(0x00);
    glcd_adddat(0x01);
    
    // unten links
    glcd_setpos(6*128+12U+temp);
    glcd_adddat(0xF2);
    glcd_adddat(0x80);
    glcd_adddat(0x80);
    glcd_adddat(0x80);
    glcd_adddat(0x00);
    glcd_adddat(0x00);
    glcd_adddat(0x80);
    
    // unten rechts
    glcd_setpos(6*128+46U+temp);
    glcd_adddat(0x80);
    glcd_adddat(0x00);
    glcd_adddat(0x00);
    glcd_adddat(0x80);
    glcd_adddat(0x80);
    glcd_adddat(0x80);
    glcd_adddat(0xF2);
    
    // oben rechts
    glcd_setpos(2*128+46U+temp);
    glcd_adddat(0x01);
    glcd_adddat(0x00);
    glcd_adddat(0x00);
    glcd_adddat(0x01);
    glcd_adddat(0x01);
    glcd_adddat(0x01);
    glcd_adddat(0x4F);
    
    io_setBit(SPARE0_GPIO,SPARE0_Pin);
    // Bild aktualisieren
    
    glcd_drawField(iFrame); 
    iFrame = 1-iFrame;
    io_clearBit(SPARE0_GPIO,SPARE0_Pin);
}



/* Screen 1: Detaildarstellung als Text, zu Debug-Zwecken */
void screen1_init(void)
{
    setTimeout(&timer,0);
}

void screen1_update(void)
{
    // Daten aus globalen Structs, werden nur gelesen
    ctrlstat_t const * Cstat = &Glcstat;
    ctrlstat_t const * Ustat = &Glustat;
    
    
    uint8_t temp;
    uint8_t yOffset[2] = {0,64};
    static uint8_t iFrame = 0; // Momentan dargestellte Displayhaelfte
    

    // Bildschirm leeren und Ueberschrift
    printf_P(PSTR("\fRotorsteuerung"));
    
    
    /* Darstellungsteile, die doppelt vorkommen *************************************** */
    

    // Motorstatus: Bewegung oder Pause
    glcd_setpos(2U*128U+18U+yOffset[iFrame]);
    switch (Cstat->mot[iFrame].motion) 
    {
        case motor_left:
            // "<"
            glcd_putstr_P(PSTR(" \x11    "));
            break;
        
        case motor_runout_left:
            // "||" (invertiert)
            glcd_putstr_P(PSTR(" \033i \x13 \033i  "));
            break;
            
        case motor_stop: 
            // Pausenzeichen "||"
            glcd_putstr_P(PSTR("  \x13   "));
            break;

        case motor_runout_right:
            // "||" (invertiert)
            glcd_putstr_P(PSTR(" \033i \x13 \033i  "));
            break;

        case motor_right:
            // ">"
            glcd_putstr_P(PSTR("   \x10  "));
            break;
        
    }


    // Anzahl der Pulse
    glcd_setpos(5U*128U+yOffset[iFrame]);
    printf_P(PSTR(" pul%1u=%4i"),iFrame,Cstat->mot[iFrame].pulse);
        
        
    // Motorstrom
    glcd_setpos(6U*128U+yOffset[iFrame]);
    printf_P(PSTR(" cur%1u=%4i"),iFrame,Cstat->mot[iFrame].current);
        
            
    // Soll-Richtung
    if (Ustat->usrctrl == ctrl_angle)
    {
        glcd_setpos(3U*128U+yOffset[iFrame]);
        printf_P(PSTR(" dst%1u=%4i"),iFrame,Ustat->mot[iFrame].deg);
    }            
         
            
    // Ist-Richtung
    glcd_setpos(4U*128U+yOffset[iFrame]);
    printf_P(PSTR(" deg%1u=%4i"),iFrame,Cstat->mot[iFrame].deg);


    // Wenn Fehler: Soll-Richtung durch Fehlermeldung ueberschreiben        
    if (Cstat->mot[iFrame].err)
    {
        glcd_setpos(3U*128U+5U+yOffset[iFrame]);
        glcd_putdat(0xFF);
        printf_P(PSTR("\033iRotorfehl.\033i"));
    }



    /* Darstellungsteile, die nur einfach vorkommen *********************************** */

    // Busy-Zeichen (EEPROM nicht gesichert) als Animation
    if(Cstat->busy)
    {
        static char c = '/';

        if (pollTimeout(&timer))
        {
            switch (c)
            {
                case '/':  c = '-'; break;
                case '-':  c = '\\'; break;
                case '\\': c = '|'; break;
                case '|':  c = '/'; break;
            }
            setTimeout(&timer,500);
        }
        glcd_setpos(7U*128U);
        glcd_putchar(c);
    }


    // Busy-Zeichen (EEPROM nicht gesichert) als Zeichen
    glcd_setpos(0*128U+64U);
    if(Cstat->busy)
    {
        printf_P(PSTR("      \033i busy \033i\n"));
    }
    else
    {
        printf_P(PSTR("\n "));
    }


    // Aktiv-Balken
    temp = (Ustat->activeUsr)*64U;
    glcd_setpos(128U+temp);
    printf_P(PSTR(" \x12\x12\x12\x12\x12\x12\x12\x12\x12"));
    glcd_setpos(896U+temp);
    printf_P(PSTR(" \x12\x12\x12\x12\x12\x12\x12\x12\x12"));


    // Bildhaelfte aktualisieren
    glcd_drawField(iFrame);
    iFrame = 1-iFrame;
}



/* Darstellung der entsprechenden Screens */
void display_update(void)
{
   
    switch(GlSetting.screen) 
    {
        case 0:  // Fallthru
        case 1:
            // Zeigerdarstellung
            screen0_update();
            break;
        case 2:  // Fallthru
        
        case 3:
        
            // Text-Darstellung (zu Debug-Zwecken)
            screen1_update();
            break;
        default:
            error("invalid screen");
    }
}
