/*
 * rot_settings.c
 *
 * Harte und EEPROM-Einstellungen fuer die Rotorsteuerung
 *
 * 
 * Created: 27.08.2015
 *  Author: Nicolas
 */ 

#include "rot_settings.h"
#include <stdint.h>
#include <string.h>
#include "../../common/src_base/datatypes.h"
#include "../../common/src_misc/crc.h"
#include "../../common/src_eeprom/nvram.h"



// Globales Struct (SRAM-Kopie fuer alle EEPROM-Einstellungen)
rot_settings_t GlSetting = GLOBAL_SETTINGS_DEFAULT;


// Speicherplatz im EEPROM
#define EEPROM_SETTING_ADDRESS 0
#define SIZE_SETDATA (sizeof(GlSetting) )


/* Wurde durch compound-Literals fuer Default-Werte ueberfluessig */
uint_fast8_t rot_settings_init(void) 
{
    return EXIT_SUCCESS;
}



void rot_settings_calc_help_value(void)
{

}


/* Konfigurationsdaten aus EEPROM auslesen und bei OK das globale struct ueberschreiben */
uint_fast8_t rot_settings_read(void) 
{
    uint8_t buffer[SIZE_SETDATA];
    uint_fast8_t status;
    status = nvram_read(buffer,EEPROM_SETTING_ADDRESS,SIZE_SETDATA);
  
    if (status==EXIT_SUCCESS) 
    {
    	memcpy(&GlSetting,buffer,SIZE_SETDATA);
    	rot_settings_calc_help_value();
    	return EXIT_SUCCESS;
    }
    else 
    {
        return status;
    }
}



/* Konfigurationsdaten in EEPROM schreiben.
 * Wenn das schreiben fehlerhaft war, sind die alten Konfigurationsdaten im EEPROM leider 
 * verloren. */
uint_fast8_t rot_settings_write(void)
{
	return nvram_write((uint8_t *) &GlSetting,EEPROM_SETTING_ADDRESS,SIZE_SETDATA);
}




/* Konfigurationsdaten auf default-Werte setzen
 * z.B. fuer leeres EEPROM oder globalen Reset der Parameter */
uint_fast8_t rot_settings_default(void) 
{
    rot_settings_t temp = GLOBAL_SETTINGS_DEFAULT;
    GlSetting = temp;

	return EXIT_SUCCESS;
}

