/*
 * rot_variables.h
 *
 * Globale Statusvariablen fuer Rotorsteuerung
 *
 * Created: 19.08.2015 20:29:11
 *  Author: Nicolas
 */ 


#ifndef ROT_VARIABLES_H
	#define ROT_VARIABLES_H

    #include <stdint.h>
    #include <stdbool.h>


    // Benutzerkontrollmoeglichkeiten
    typedef enum
    {
        ctrl_motion, // Benutzer steuert Bewegung (rechts/links/stop)
        ctrl_angle,  // Benutzer steuert Himmelsrichtung
        ctrl_finish, // Programmteil verlassen
    } usrctrl_t ;
    
    
    
    // Bewegungszustaende eines Motors/Rotors    
    typedef enum 
    {
        motor_left         = -2,   // links
        motor_runout_left  = -1,   // auslaufen lassen
        motor_stop         =  0,   // Stillstand
        motor_runout_right =  1,   // auslaufen lassen
        motor_right        =  2,   // rechts
    } motion_t;
  


    // Daten je Motor/Rotor
    typedef struct 
    {
        // Aktuelle Daten, die von Sensoren ermittelt/gezhlt/gemessen werden
        uint16_t adc;           // Gefilterte ADC-Werte Shunt (gemessen)
        int16_t pulse;          // Flanken Motorgeber (gemessen/gezaehlt)
        
        // Aktuelle Daten, die aus Eingangsdaten berechnet werden
        uint16_t current;       // Phasenstrom [mA]
        int16_t deg;            // Winkel [] (Offset: 360)
        
        // Selten geaenderte Daten, die sich aus der Konfiguration ergeben
        int16_t degMax;         // Obere Grenze inkl. Ueberlauf
        int16_t degMin;
        
        // Zustandsgroessen
        motion_t motion;
        bool err;               // Nur Cstat: Motorfehler; Motor darf nur noch mit 
                                // Handbetrieb gesteuert werden
    } motordat_t;


    // Status (Soll und Ist)
    typedef struct 
    {
        motordat_t mot[2];      // Daten der einzelnen Motoren
        uint8_t activeMot;      // aktiver Motor ("es kann nur einen geben")
        uint8_t activeUsr;      // Nur Ustat: aktive Benutzerseite 
        usrctrl_t usrctrl;      // Winkelsteuerung, start/stop/pause, Programmende
        bool busy;              // Nur Cstat: EEPROM nicht fertig, Geraet darf nicht 
                                // ausgeschaltet werden        
    } ctrlstat_t;


    // Ist-Status (Global currect state)
    extern ctrlstat_t Glcstat;        
        
    #define GL_CSTAT_INIT \
    {\
        .mot[0] = {.adc = 0U, .deg = 0, .motion = motor_stop, .err=0},\
        .mot[1] = {.adc = 0U, .deg = 0, .motion = motor_stop, .err=0},\
        .activeMot = 0,\
        .usrctrl = ctrl_motion,\
    }
    

    // Soll-Status/Benutzerkontrolldaten
    extern ctrlstat_t Glustat;
    
    #define GL_USTAT_INIT \
    {\
        .mot[0] = {.motion = motor_stop},\
        .mot[1] = {.motion = motor_stop},\
        .activeMot = 0,\
        .usrctrl = ctrl_motion,\
    }
    
#endif

